@@ -321,6 +321,8 @@ char      	usecolors  = 1;  /* boolean: colors for high occupation  */
 char		threadview = 0;	 /* boolean: show individual threads     */
 char      	calcpss    = 0;  /* boolean: read/calculate process PSS  */
 char      	getwchan   = 0;  /* boolean: obtain wchan string         */
+char      	rmspaces   = 0;  /* boolean: remove spaces from command  */
+		                 /* name in case of parseable output     */
 
 unsigned short	hertz;
 unsigned int	pagesize;
@@ -363,6 +365,7 @@ void do_maxintf(char *, char *);
 void do_maxifb(char *, char *);
 void do_maxnfsm(char *, char *);
 void do_maxcont(char *, char *);
+void do_maxnuma(char *, char *);
 void do_colinfo(char *, char *);
 void do_colalmost(char *, char *);
 void do_colcrit(char *, char *);
@@ -374,6 +377,8 @@ void do_owncplline(char *, char *);
 void do_ownmemline(char *, char *);
 void do_ownswpline(char *, char *);
 void do_ownpagline(char *, char *);
+void do_ownmemnumaline(char *, char *);
+void do_owncpunumaline(char *, char *);
 void do_owndskline(char *, char *);
 void do_ownnettransportline(char *, char *);
 void do_ownnetnetline(char *, char *);
@@ -411,6 +416,7 @@ static struct {
 	{	"maxlineifb",		do_maxifb,		0, },
 	{	"maxlinenfsm",		do_maxnfsm,		0, },
 	{	"maxlinecont",		do_maxcont,		0, },
+	{	"maxlinenuma",		do_maxnuma,		0, },
 	{	"colorinfo",		do_colinfo,		0, },
 	{	"coloralmost",		do_colalmost,		0, },
 	{	"colorcritical",	do_colcrit,		0, },
@@ -421,6 +427,8 @@ static struct {
 	{	"ownmemline",		do_ownmemline,		0, },
 	{	"ownswpline",		do_ownswpline,		0, },
 	{	"ownpagline",		do_ownpagline,		0, },
+	{	"ownmemnumaline",	do_ownmemnumaline,	0, },
+	{	"ownnumacpuline",	do_owncpunumaline,	0, },
 	{	"owndskline",		do_owndskline,		0, },
 	{	"ownnettrline",		do_ownnettransportline,	0, },
 	{	"ownnetnetline",	do_ownnetnetline,	0, },
@@ -594,6 +602,10 @@ main(int argc, char *argv[])
 				getwchan = 1;
 				break;
 
+                           case MRMSPACES:	/* remove spaces from command */
+				rmspaces = 1;
+				break;
+
 			   default:		/* gather other flags */
 				flaglist[i++] = c;
 			}
@@ -1071,6 +1083,8 @@ prusage(char *myname)
 	                MCALCPSS);
 	printf("\t  -%c  determine WCHAN (string) per thread\n", MGETWCHAN);
 	printf("\t  -P  generate parseable output for specified label(s)\n");
+	printf("\t  -%c  no spaces in parseable output for command (line)\n",
+			MRMSPACES);
 	printf("\t  -L  alternate line length (default 80) in case of "
 			"non-screen output\n");
 
