@@ -28,13 +28,18 @@
 /*
 ** memory-size formatting possibilities
 */
-#define	ANYFORMAT	0
+#define	BFORMAT		0
 #define	KBFORMAT	1
-#define	MBFORMAT	2
-#define	GBFORMAT	3
-#define	TBFORMAT	4
-#define	PBFORMAT	5
-#define	OVFORMAT	9
+#define	KBFORMAT_INT	2
+#define	MBFORMAT	3
+#define	MBFORMAT_INT	4
+#define	GBFORMAT	5
+#define	GBFORMAT_INT	6
+#define	TBFORMAT	7
+#define	TBFORMAT_INT	8
+#define	PBFORMAT	9
+#define	PBFORMAT_INT	10
+#define	OVFORMAT	11
 
 typedef	long long	count_t;
 
@@ -83,6 +88,7 @@ extern char		calcpss;
 extern char		getwchan;
 extern char		rawname[];
 extern char		rawreadflag;
+extern char		rmspaces;
 extern time_t		begintime, endtime, cursortime;	// epoch or time in day
 extern char		flaglist[];
 extern struct visualize vis;
@@ -164,8 +170,9 @@ int		intfcompar(const void *, const void *);
 int		ifbcompar(const void *, const void *);
 int		nfsmcompar(const void *, const void *);
 int		contcompar(const void *, const void *);
+int		memnumacompar(const void *, const void *);
+int		cpunumacompar(const void *, const void *);
 
-count_t		subcount(count_t, count_t);
 int  		rawread(void);
 char		rawwrite (time_t, int,
 		            struct devtstat *, struct sstat *,
