@@ -1484,9 +1484,9 @@ memline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	unsigned int	mbadness, sbadness;
 
 	if (membadness)
-		mbadness = ((ss->mem.physmem  - ss->mem.freemem -
-	                     ss->mem.cachemem - ss->mem.buffermem)
-	                               * 100.0 / ss->mem.physmem) 
+		mbadness = ((ss->mem.physmem  - ss->mem.freemem 
+	                     - ss->mem.cachemem - ss->mem.buffermem
+	                     + ss->mem.shmem) * 100.0 / ss->mem.physmem) 
 	                               * 100   / membadness;
 	else
 		mbadness = 0;
@@ -1521,7 +1521,7 @@ memline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 swaphead(int osvers, int osrel, int ossub)
 {
-	printf("pagescan/s   swapin/s  swapout/s      "
+	printf("pagescan/s  swapin/s swapout/s oomkill"
 	       "  commitspc  commitlim   _swap_");
 }
 
@@ -1552,10 +1552,11 @@ swapline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 
 	preprint(badness);
 
-	printf("%10.2lf  %9.2lf  %9.2lf       %9lluM %9lluM",
+	printf("%10.2lf %9.2lf %9.2lf %7lld %9lluM %9lluM",
 		(double)ss->mem.pgscans / deltasec,
 		(double)ss->mem.swins   / deltasec,
 		(double)ss->mem.swouts  / deltasec,
+		        ss->mem.oomkills,
 		        ss->mem.committed * (pagesize / 1024) / 1024,
 		        ss->mem.commitlim * (pagesize / 1024) / 1024);
 
@@ -1695,8 +1696,10 @@ gendskline(struct sstat *ss, char *tstamp, char selector)
 		char	*pn;
 		int	len;
 
-		if ( (iotot = dp->nread + dp->nwrite) == 0 &&
- 		     !firstcall && !allresources             )
+		iotot = dp->nread + dp->nwrite +
+		             (dp->ndisc != -1 ? dp->ndisc : 0);
+
+		if (iotot == 0 && !firstcall && !allresources)
 			continue;	/* no activity on this disk */
 
 		/*
@@ -1921,12 +1924,12 @@ ibline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 			continue;
 
 		/*
-		** convert byte-transfers to bit-transfers     (*       8)
-		** convert bit-transfers  to megabit-transfers (/ 1000000)
+		** convert byte-transfers to bit-transfers     (*          8)
+		** convert bit-transfers  to gigabit-transfers (/ 1000000000)
 		** per second
 		*/
-		ival = ss->ifb.ifb[i].rcvb*ss->ifb.ifb[i].lanes/125000/deltasec;
-		oval = ss->ifb.ifb[i].sndb*ss->ifb.ifb[i].lanes/125000/deltasec;
+		ival = ss->ifb.ifb[i].rcvb*ss->ifb.ifb[i].lanes/125000000/deltasec;
+		oval = ss->ifb.ifb[i].sndb*ss->ifb.ifb[i].lanes/125000000/deltasec;
 
 		/*
 		** calculate busy-percentage for port
@@ -2853,7 +2856,7 @@ struct pridef pridef[] =
    {0,  "n",  't',  tcphead,	tcpline,  	"tcp        (general)",   },
    {0,  "n",  'T',  TCPhead,	TCPline,  	"tcp        (errors)",    },
 #if	HTTPSTATS
-   {0,  "n",  'h',  httphead,	httpline,  	"HTTP activity",          },
+   {0,  "n",  'o',  httphead,	httpline,  	"HTTP activity",          },
 #endif
    {0,  "",   'O',  topchead,	topcline,  	"top-3 processes cpu",    },
    {0,  "",   'G',  topmhead,	topmline,  	"top-3 processes memory", },
