@@ -12,7 +12,7 @@
 ** Date:        November 1996
 ** LINUX-port:  June 2000
 ** --------------------------------------------------------------------------
-** Copyright (C) 2000-2010 Gerlof Langeveld
+** Copyright (C) 2000-2021 Gerlof Langeveld
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -28,144 +28,6 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
-**
-** $Log: deviate.c,v $
-** Revision 1.45  2010/10/23 14:02:03  gerlof
-** Show counters for total number of running and sleep (S and D) threads.
-**
-** Revision 1.44  2010/05/18 19:19:43  gerlof
-** Introduce CPU frequency and scaling (JC van Winkel).
-**
-** Revision 1.43  2010/04/23 12:19:35  gerlof
-** Modified mail-address in header.
-**
-** Revision 1.42  2010/03/04 10:52:08  gerlof
-** Support I/O-statistics on logical volumes and MD devices.
-**
-** Revision 1.41  2009/12/31 11:34:21  gerlof
-** Sanity-check to bypass kernel-bug showing 497 days of CPU-consumption.
-**
-** Revision 1.40  2009/12/17 11:58:25  gerlof
-** Gather and display new counters: dirty cache and guest cpu usage.
-**
-** Revision 1.39  2008/02/25 14:51:18  gerlof
-** Experimental code for HTTP-statistics.
-**
-** Revision 1.38  2008/01/07 11:33:43  gerlof
-** Cosmetic changes.
-**
-** Revision 1.37  2008/01/07 10:17:24  gerlof
-** Implement possibility to make summaries.
-**
-** Revision 1.36  2007/11/05 12:13:16  gerlof
-** Match processes not only on pid, but also on start time.
-**
-** Revision 1.35  2007/11/05 11:42:47  gerlof
-** Bug-solution for new-process indicator on 64-bits machines.
-**
-** Revision 1.34  2007/08/17 09:44:59  gerlof
-** Experimental: gather info about HTTP statistics.
-**
-** Revision 1.33  2007/08/16 11:59:32  gerlof
-** Add support for atopsar reporting.
-** Concerns addition of lots of counters.
-**
-** Revision 1.32  2007/07/03 09:01:07  gerlof
-** Support Apache-statistics.
-**
-** Revision 1.31  2007/03/20 13:02:03  gerlof
-** Introduction of variable supportflags.
-**
-** Revision 1.30  2007/03/20 11:18:57  gerlof
-** Add counter for cancelled writes.
-**
-** Revision 1.29  2007/02/13 09:21:04  gerlof
-** Removed external declarations.
-**
-** Revision 1.28  2007/01/22 08:28:18  gerlof
-** Support steal-time from /proc/stat.
-**
-** Revision 1.27  2007/01/18 10:43:18  gerlof
-** Support for network-interface busy-percentage (speed and duplex).
-**
-** Revision 1.26  2006/11/13 13:47:26  gerlof
-** Implement load-average counters, context-switches and interrupts.
-**
-** Revision 1.25  2006/02/07 06:45:33  gerlof
-** Removed swap-counter.
-**
-** Revision 1.24  2006/01/30 09:13:33  gerlof
-** Extend memory counters (a.o. page scans).
-**
-** Revision 1.23  2005/10/31 12:45:29  gerlof
-** Support account-record version 3 (used by Mandriva).
-**
-** Revision 1.22  2005/10/21 09:49:38  gerlof
-** Per-user accumulation of resource consumption.
-**
-** Revision 1.21  2004/12/14 15:05:47  gerlof
-** Implementation of patch-recognition for disk and network-statistics.
-**
-** Revision 1.20  2004/10/28 08:30:51  gerlof
-** New counter: vm committed space
-**
-** Revision 1.19  2004/09/24 10:02:01  gerlof
-** Wrong cpu-numbers for system level statistics.
-**
-** Revision 1.18  2004/09/02 10:49:18  gerlof
-** Added sleep-average to process-info.
-**
-** Revision 1.17  2004/08/31 13:27:04  gerlof
-** Add new info for threading.
-**
-** Revision 1.16  2004/05/07 05:49:40  gerlof
-** *** empty log message ***
-**
-** Revision 1.15  2004/05/06 09:46:55  gerlof
-** Ported to kernel-version 2.6.
-**
-** Revision 1.14  2003/07/07 09:26:33  gerlof
-** Cleanup code (-Wall proof).
-**
-** Revision 1.13  2003/07/03 11:17:49  gerlof
-** Corrected calculations for exited processes.
-**
-** Revision 1.12  2003/06/30 11:30:57  gerlof
-** Enlarge counters to 'long long'.
-**
-** Revision 1.11  2003/06/24 06:21:12  gerlof
-** Limit number of system resource lines.
-**
-** Revision 1.10  2003/01/24 14:20:16  gerlof
-** If possible, also show commandline when process has exited.
-**
-** Revision 1.9  2002/09/16 08:58:08  gerlof
-** Add indicator for newly created processes.
-**
-** Revision 1.8  2002/08/27 04:47:46  gerlof
-** Minor comment updates.
-**
-** Revision 1.7  2002/07/24 11:12:20  gerlof
-** Redesigned to ease porting to other UNIX-platforms.
-**
-** Revision 1.6  2002/07/10 04:59:37  root
-** Counters pin/pout renamed to swin/swout (Linux conventions).
-**
-** Revision 1.5  2002/01/22 13:39:20  gerlof
-** Support for number of cpu's.
-**
-** Revision 1.4  2001/11/22 08:33:10  gerlof
-** Add priority per process.
-**
-** Revision 1.3  2001/11/07 09:18:22  gerlof
-** Use /proc instead of /dev/kmem for process-level statistics.
-**
-** Revision 1.2  2001/10/03 08:58:41  gerlof
-** Improved subtraction which is overflow-proof
-**
-** Revision 1.1  2001/10/02 10:43:23  gerlof
-** Initial revision
-**
 */
 
 #include <sys/types.h>
@@ -187,10 +49,10 @@
 #include "photoproc.h"
 #include "photosyst.h"
 
-#define	MAX32BITVAL	0x100000000LL
-
-static void calcdiff(struct tstat *, struct tstat *, struct tstat *,
-							char, count_t);
+static 		void calcdiff(struct tstat *, const struct tstat *,
+		                              const struct tstat *,
+		                              char, count_t);
+static inline	count_t subcount(count_t, count_t);
 
 /*
 ** calculate the process-activity during the last sample
@@ -203,7 +65,7 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
 {
 	register int		c, d, pall=0, pact=0;
 	register struct tstat	*curstat, *devstat, *thisproc;
-	struct tstat		prestat;
+	struct tstat		prestat, *pprestat;
 	struct pinfo		*pinfo;
 	count_t			totusedcpu;
 	char			hashtype = 'p';
@@ -294,6 +156,7 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
  				** no activity for task
 				*/
 				curstat->gen.wasinactive = 1;
+				pprestat = curstat;
 			}
  			else
 			{
@@ -303,6 +166,7 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
 				** the database with the current sample
 				*/
 				prestat 	= pinfo->tstat;
+				pprestat	= &prestat;
 				pinfo->tstat 	= *curstat;
 
 				curstat->gen.wasinactive = 0;
@@ -331,6 +195,7 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
 			** last interval
 			*/
 			memset(&prestat, 0, sizeof(prestat));
+			pprestat = &prestat;
 
 			curstat->gen.wasinactive = 0;
 			devtstat->ntaskactive++;
@@ -369,7 +234,7 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
 		/*
 		** do the difference calculations
 		*/
-		calcdiff(devstat, curstat, &prestat, newtask, totusedcpu);
+		calcdiff(devstat, curstat, pprestat, newtask, totusedcpu);
 	}
 
 	/*
@@ -558,17 +423,21 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
 ** the previous sample for a task
 */
 static void
-calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
-	                                      char newtask, count_t totusedcpu)
+calcdiff(struct tstat *devstat, const struct tstat *curstat,
+                                const struct tstat *prestat,
+	                        char newtask, count_t totusedcpu)
 {
 	/*
- 	** for inactive tasks, set all counters to zero
+ 	** for inactive tasks, set all counters to zero to avoid calculating
+	** the deviations (after all, there are no deviations)
 	*/
 	if (curstat->gen.wasinactive)
+	{
 		memset(devstat, 0, sizeof *devstat);
+	}
 
 	/*
-	** copy all static values from the current task settings
+	** copy all STATIC values from the current task settings
 	*/
 	devstat->gen          = curstat->gen;
 
@@ -618,11 +487,15 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 
 	/*
  	** for inactive tasks, only the static values had to be copied, while
-	** all use counters have been set to zero
+	** all use counters have already been set to zero
 	*/
 	if (curstat->gen.wasinactive)
 		return;
 
+	/*
+	** calculate deviations for tasks that were really active
+	** (i.e. modified) during the sample
+	*/
 	devstat->cpu.stime  = 
 		subcount(curstat->cpu.stime, prestat->cpu.stime);
 	devstat->cpu.utime  =
@@ -642,9 +515,7 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 
 	devstat->cpu.rundelay  =
 		subcount(curstat->cpu.rundelay, prestat->cpu.rundelay);
-	/*
-	** do further calculations
-	*/
+
 	devstat->dsk.rio    =
 		subcount(curstat->dsk.rio, prestat->dsk.rio);
 	devstat->dsk.rsz    =
@@ -826,6 +697,10 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 	dev->mem.vmwballoon	= cur->mem.vmwballoon;
 	dev->mem.zfsarcsize	= cur->mem.zfsarcsize;
+	dev->mem.ksmsharing	= cur->mem.ksmsharing;
+	dev->mem.ksmshared 	= cur->mem.ksmshared;
+	dev->mem.zswstored	= cur->mem.zswstored;
+	dev->mem.zswtotpool	= cur->mem.zswtotpool;
 
 	dev->mem.swouts		= subcount(cur->mem.swouts,  pre->mem.swouts);
 	dev->mem.swins		= subcount(cur->mem.swins,   pre->mem.swins);
@@ -834,7 +709,63 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 	dev->mem.allocstall	= subcount(cur->mem.allocstall,
 				                         pre->mem.allocstall);
 
-	dev->psi          	= cur->psi;
+	if (cur->mem.oomkills != -1)
+		dev->mem.oomkills = subcount(cur->mem.oomkills, pre->mem.oomkills);
+	else
+		dev->mem.oomkills = -1;
+
+	dev->mem.compactstall	= subcount(cur->mem.compactstall,
+				                         pre->mem.compactstall);
+	dev->mem.numamigrate	= subcount(cur->mem.numamigrate, pre->mem.numamigrate);
+	dev->mem.pgmigrate	= subcount(cur->mem.pgmigrate,   pre->mem.pgmigrate);
+
+	dev->memnuma.nrnuma     = cur->memnuma.nrnuma;
+
+	for (i=0; i < dev->memnuma.nrnuma; i++)
+	{
+		dev->memnuma.numa[i].totmem      = cur->memnuma.numa[i].totmem;
+		dev->memnuma.numa[i].freemem     = cur->memnuma.numa[i].freemem;
+		dev->memnuma.numa[i].filepage    = cur->memnuma.numa[i].filepage;
+		dev->memnuma.numa[i].active      = cur->memnuma.numa[i].active;
+		dev->memnuma.numa[i].inactive    = cur->memnuma.numa[i].inactive;
+		dev->memnuma.numa[i].dirtymem    = cur->memnuma.numa[i].dirtymem;
+		dev->memnuma.numa[i].shmem       = cur->memnuma.numa[i].shmem;
+		dev->memnuma.numa[i].slabmem     = cur->memnuma.numa[i].slabmem;
+		dev->memnuma.numa[i].slabreclaim = cur->memnuma.numa[i].slabreclaim;
+		dev->memnuma.numa[i].tothp       = cur->memnuma.numa[i].tothp;
+		dev->memnuma.numa[i].frag        = cur->memnuma.numa[i].frag;
+	}
+
+	dev->cpunuma.nrnuma = cur->cpunuma.nrnuma;
+
+	if (dev->cpunuma.nrnuma > 1)
+	{
+		for (i=0; i < dev->cpunuma.nrnuma; i++)
+		{
+			dev->cpunuma.numa[i].nrcpu  = cur->cpunuma.numa[i].nrcpu;
+
+			dev->cpunuma.numa[i].utime  = subcount(cur->cpunuma.numa[i].utime,
+								pre->cpunuma.numa[i].utime);
+			dev->cpunuma.numa[i].ntime  = subcount(cur->cpunuma.numa[i].ntime,
+								pre->cpunuma.numa[i].ntime);
+			dev->cpunuma.numa[i].stime  = subcount(cur->cpunuma.numa[i].stime,
+								pre->cpunuma.numa[i].stime);
+			dev->cpunuma.numa[i].itime  = subcount(cur->cpunuma.numa[i].itime,
+								pre->cpunuma.numa[i].itime);
+			dev->cpunuma.numa[i].wtime  = subcount(cur->cpunuma.numa[i].wtime,
+								pre->cpunuma.numa[i].wtime);
+			dev->cpunuma.numa[i].Itime  = subcount(cur->cpunuma.numa[i].Itime,
+								pre->cpunuma.numa[i].Itime);
+			dev->cpunuma.numa[i].Stime  = subcount(cur->cpunuma.numa[i].Stime,
+								pre->cpunuma.numa[i].Stime);
+			dev->cpunuma.numa[i].steal  = subcount(cur->cpunuma.numa[i].steal,
+								pre->cpunuma.numa[i].steal);
+			dev->cpunuma.numa[i].guest  = subcount(cur->cpunuma.numa[i].guest,
+								pre->cpunuma.numa[i].guest);
+		}
+	}
+
+	dev->psi = cur->psi;
 
 	if (cur->psi.present)
 	{
@@ -934,18 +865,10 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 	/*
 	** calculate deviations for interfaces
-	**
-	** refresh all interface properties
 	*/
-        regainrootprivs();	/* get root privileges      */
-
-	initifprop();		/* refresh interface info   */
-
-	if (! droprootprivs())  /* drop setuid-root privs   */
-		mcleanstop(42, "failed to drop root privs\n");
-
 	for (i=0; cur->intf.intf[i].name[0]; i++)
 	{
+		// fill current properties for each valid interface
 		strcpy(ifprop.name, cur->intf.intf[i].name);
 
 		getifprop(&ifprop);
@@ -1081,10 +1004,10 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 		dev->dsk.dsk[i].nread  = subcount(cur->dsk.dsk[i].nread,
 		                                  pre->dsk.dsk[j].nread);
-		dev->dsk.dsk[i].nwrite = subcount(cur->dsk.dsk[i].nwrite,
-		                                  pre->dsk.dsk[j].nwrite);
 		dev->dsk.dsk[i].nrsect = subcount(cur->dsk.dsk[i].nrsect,
 		                                  pre->dsk.dsk[j].nrsect);
+		dev->dsk.dsk[i].nwrite = subcount(cur->dsk.dsk[i].nwrite,
+		                                  pre->dsk.dsk[j].nwrite);
 		dev->dsk.dsk[i].nwsect = subcount(cur->dsk.dsk[i].nwsect,
 		                                  pre->dsk.dsk[j].nwsect);
 		dev->dsk.dsk[i].io_ms  = subcount(cur->dsk.dsk[i].io_ms,
@@ -1092,6 +1015,19 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 		dev->dsk.dsk[i].avque  = subcount(cur->dsk.dsk[i].avque,
 		                                  pre->dsk.dsk[j].avque);
 
+		if (cur->dsk.dsk[i].ndisc != -1)	// discards supported?
+		{
+			dev->dsk.dsk[i].ndisc  = subcount(cur->dsk.dsk[i].ndisc,
+		                                          pre->dsk.dsk[j].ndisc);
+			dev->dsk.dsk[i].ndsect = subcount(cur->dsk.dsk[i].ndsect,
+		                                          pre->dsk.dsk[j].ndsect);
+		}
+		else
+		{
+			dev->dsk.dsk[i].ndisc  = -1;
+			dev->dsk.dsk[i].ndsect = 0;
+		}
+
 		/*
 		** determine new j
 		*/
@@ -1137,10 +1073,10 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 		dev->dsk.mdd[i].nread  = subcount(cur->dsk.mdd[i].nread,
 		                                  pre->dsk.mdd[j].nread);
-		dev->dsk.mdd[i].nwrite = subcount(cur->dsk.mdd[i].nwrite,
-		                                  pre->dsk.mdd[j].nwrite);
 		dev->dsk.mdd[i].nrsect = subcount(cur->dsk.mdd[i].nrsect,
 		                                  pre->dsk.mdd[j].nrsect);
+		dev->dsk.mdd[i].nwrite = subcount(cur->dsk.mdd[i].nwrite,
+		                                  pre->dsk.mdd[j].nwrite);
 		dev->dsk.mdd[i].nwsect = subcount(cur->dsk.mdd[i].nwsect,
 		                                  pre->dsk.mdd[j].nwsect);
 		dev->dsk.mdd[i].io_ms  = subcount(cur->dsk.mdd[i].io_ms,
@@ -1148,6 +1084,19 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 		dev->dsk.mdd[i].avque  = subcount(cur->dsk.mdd[i].avque,
 		                                  pre->dsk.mdd[j].avque);
 
+		if (cur->dsk.mdd[i].ndisc != -1)	// discards supported?
+		{
+			dev->dsk.mdd[i].ndisc  = subcount(cur->dsk.mdd[i].ndisc,
+		                                          pre->dsk.mdd[j].ndisc);
+			dev->dsk.mdd[i].ndsect = subcount(cur->dsk.mdd[i].ndsect,
+		                                          pre->dsk.mdd[j].ndsect);
+		}
+		else
+		{
+			dev->dsk.mdd[i].ndisc  = -1;
+			dev->dsk.mdd[i].ndsect = 0;
+		}
+
 		/*
 		** determine new j
 		*/
@@ -1193,10 +1142,10 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 		dev->dsk.lvm[i].nread  = subcount(cur->dsk.lvm[i].nread,
 		                                  pre->dsk.lvm[j].nread);
-		dev->dsk.lvm[i].nwrite = subcount(cur->dsk.lvm[i].nwrite,
-		                                  pre->dsk.lvm[j].nwrite);
 		dev->dsk.lvm[i].nrsect = subcount(cur->dsk.lvm[i].nrsect,
 		                                  pre->dsk.lvm[j].nrsect);
+		dev->dsk.lvm[i].nwrite = subcount(cur->dsk.lvm[i].nwrite,
+		                                  pre->dsk.lvm[j].nwrite);
 		dev->dsk.lvm[i].nwsect = subcount(cur->dsk.lvm[i].nwsect,
 		                                  pre->dsk.lvm[j].nwsect);
 		dev->dsk.lvm[i].io_ms  = subcount(cur->dsk.lvm[i].io_ms,
@@ -1204,6 +1153,19 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 		dev->dsk.lvm[i].avque  = subcount(cur->dsk.lvm[i].avque,
 		                                  pre->dsk.lvm[j].avque);
 
+		if (cur->dsk.lvm[i].ndisc != -1)	// discards supported?
+		{
+			dev->dsk.lvm[i].ndisc  = subcount(cur->dsk.lvm[i].ndisc,
+		                                          pre->dsk.lvm[j].ndisc);
+			dev->dsk.lvm[i].ndsect = subcount(cur->dsk.lvm[i].ndsect,
+		                                          pre->dsk.lvm[j].ndsect);
+		}
+		else
+		{
+			dev->dsk.lvm[i].ndisc  = -1;
+			dev->dsk.lvm[i].ndsect = 0;
+		}
+
 		/*
 		** determine new j
 		*/
@@ -1546,6 +1508,7 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 		tot->mem.swins		+= new->mem.swins;
 		tot->mem.pgscans	+= new->mem.pgscans;
 		tot->mem.allocstall	+= new->mem.allocstall;
+		tot->mem.compactstall	+= new->mem.compactstall;
 		break;
 
 	   case 'n':	/* accumulate network-related counters */
@@ -1716,11 +1679,18 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 			strcpy(tot->dsk.dsk[i].name, new->dsk.dsk[i].name);
 	
 			tot->dsk.dsk[i].nread  += new->dsk.dsk[i].nread;
-			tot->dsk.dsk[i].nwrite += new->dsk.dsk[i].nwrite;
 			tot->dsk.dsk[i].nrsect += new->dsk.dsk[i].nrsect;
+			tot->dsk.dsk[i].nwrite += new->dsk.dsk[i].nwrite;
 			tot->dsk.dsk[i].nwsect += new->dsk.dsk[i].nwsect;
 			tot->dsk.dsk[i].io_ms  += new->dsk.dsk[i].io_ms;
 			tot->dsk.dsk[i].avque  += new->dsk.dsk[i].avque;
+
+			if (new->dsk.dsk[i].ndisc != -1) // discards?
+			{
+			    tot->dsk.dsk[i].ndisc  += new->dsk.dsk[i].ndisc;
+			    tot->dsk.dsk[i].ndsect += new->dsk.dsk[i].ndsect;
+			}
+			
 		}
 	
 		tot->dsk.dsk[i].name[0] = '\0';
@@ -1731,11 +1701,17 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 			strcpy(tot->dsk.lvm[i].name, new->dsk.lvm[i].name);
 	
 			tot->dsk.lvm[i].nread  += new->dsk.lvm[i].nread;
-			tot->dsk.lvm[i].nwrite += new->dsk.lvm[i].nwrite;
 			tot->dsk.lvm[i].nrsect += new->dsk.lvm[i].nrsect;
+			tot->dsk.lvm[i].nwrite += new->dsk.lvm[i].nwrite;
 			tot->dsk.lvm[i].nwsect += new->dsk.lvm[i].nwsect;
 			tot->dsk.lvm[i].io_ms  += new->dsk.lvm[i].io_ms;
 			tot->dsk.lvm[i].avque  += new->dsk.lvm[i].avque;
+
+			if (new->dsk.lvm[i].ndisc != -1) // discards?
+			{
+			    tot->dsk.lvm[i].ndisc  += new->dsk.lvm[i].ndisc;
+			    tot->dsk.lvm[i].ndsect += new->dsk.lvm[i].ndsect;
+			}
 		}
 	
 		tot->dsk.lvm[i].name[0] = '\0';
@@ -1746,11 +1722,17 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 			strcpy(tot->dsk.mdd[i].name, new->dsk.mdd[i].name);
 	
 			tot->dsk.mdd[i].nread  += new->dsk.mdd[i].nread;
-			tot->dsk.mdd[i].nwrite += new->dsk.mdd[i].nwrite;
 			tot->dsk.mdd[i].nrsect += new->dsk.mdd[i].nrsect;
+			tot->dsk.mdd[i].nwrite += new->dsk.mdd[i].nwrite;
 			tot->dsk.mdd[i].nwsect += new->dsk.mdd[i].nwsect;
 			tot->dsk.mdd[i].io_ms  += new->dsk.mdd[i].io_ms;
 			tot->dsk.mdd[i].avque  += new->dsk.mdd[i].avque;
+
+			if (new->dsk.mdd[i].ndisc != -1) // discards?
+			{
+			    tot->dsk.mdd[i].ndisc  += new->dsk.lvm[i].ndisc;
+			    tot->dsk.mdd[i].ndsect += new->dsk.lvm[i].ndsect;
+			}
 		}
 	
 		tot->dsk.mdd[i].name[0] = '\0';
@@ -1762,13 +1744,17 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 
 /*
 ** Generic function to subtract two counters taking into 
-** account the possibility of overflow of a 32-bit kernel-counter.
+** account the possibility that the counter is invalid
+** (i.e. non-existing).
 */
-count_t
+static inline count_t
 subcount(count_t newval, count_t oldval)
 {
-	if (newval >= oldval)
+	if (newval == -1)	// invalid counter?
+		return -1;
+
+	if (newval >= oldval)	// normal situation
 		return newval - oldval;
-	else
-		return MAX32BITVAL + newval - oldval;
+	else			// counter seems to be reset
+		return newval;
 }
