@@ -28,127 +28,7 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
-**
-** $Log: photosyst.c,v $
-** Revision 1.38  2010/11/19 07:40:40  gerlof
-** Support of virtual disk vd... (kvm).
-**
-** Revision 1.37  2010/11/14 06:42:18  gerlof
-** After opening /proc/cpuinfo, the file descriptor was not closed any more.
-**
-** Revision 1.36  2010/10/23 14:09:50  gerlof
-** Add support for mmcblk disks (MMC/SD cardreaders)
-** Credits: Anssi Hannula
-**
-** Revision 1.35  2010/05/18 19:20:30  gerlof
-** Introduce CPU frequency and scaling (JC van Winkel).
-**
-** Revision 1.34  2010/04/23 12:19:35  gerlof
-** Modified mail-address in header.
-**
-** Revision 1.33  2010/03/04 10:58:05  gerlof
-** Added recognition of device-type /dev/fio...
-**
-** Revision 1.32  2010/03/04 10:52:47  gerlof
-** Support I/O-statistics on logical volumes and MD devices.
-**
-** Revision 1.31  2009/12/17 11:59:16  gerlof
-** Gather and display new counters: dirty cache and guest cpu usage.
-**
-** Revision 1.30  2008/02/25 13:47:00  gerlof
-** Experimental code for HTTP statistics.
-**
-** Revision 1.29  2007/08/17 09:45:44  gerlof
-** Experimental: gather info about HTTP statistics.
-**
-** Revision 1.28  2007/08/16 12:00:49  gerlof
-** Add support for atopsar reporting.
-** Gather more counters, mainly related to networking.
-**
-** Revision 1.27  2007/07/03 09:01:56  gerlof
-** Support Apache-statistics.
-**
-** Revision 1.26  2007/02/13 10:32:28  gerlof
-** Removal of external declarations.
-**
-** Revision 1.25  2007/02/13 09:29:57  gerlof
-** Removal of external declarations.
-**
-** Revision 1.24  2007/01/22 14:57:12  gerlof
-** Support of special disks used by virtual machines.
-**
-** Revision 1.23  2007/01/22 08:28:50  gerlof
-** Support steal-time from /proc/stat.
-**
-** Revision 1.22  2006/11/13 13:48:20  gerlof
-** Implement load-average counters, context-switches and interrupts.
-**
-** Revision 1.21  2006/01/30 09:14:16  gerlof
-** Extend memory counters (a.o. page scans).
-**
-** Revision 1.20  2005/10/21 09:50:08  gerlof
-** Per-user accumulation of resource consumption.
-**
-** Revision 1.19  2004/10/28 08:31:23  gerlof
-** New counter: vm committed space
-**
-** Revision 1.18  2004/09/24 10:02:35  gerlof
-** Wrong cpu-numbers for system level statistics.
-**
-** Revision 1.17  2004/05/07 05:27:37  gerlof
-** Recognize new disk-names and support of diskname-modification.
-**
-** Revision 1.16  2004/05/06 09:53:31  gerlof
-** Skip statistics of ram-disks.
-**
-** Revision 1.15  2004/05/06 09:46:14  gerlof
-** Ported to kernel-version 2.6.
-**
-** Revision 1.14  2003/07/08 13:53:21  gerlof
-** Cleanup code.
-**
-** Revision 1.13  2003/07/07 09:27:06  gerlof
-** Cleanup code (-Wall proof).
-**
-** Revision 1.12  2003/06/30 11:30:37  gerlof
-** Enlarge counters to 'long long'.
-**
-** Revision 1.11  2003/06/24 06:21:40  gerlof
-** Limit number of system resource lines.
-**
-** Revision 1.10  2003/01/17 14:23:05  root
-** Change-directory to /proc to optimize opening /proc-files
-** via relative path-names i.s.o. absolute path-names.
-**
-** Revision 1.9  2003/01/14 07:50:26  gerlof
-** Consider IPv6 counters on IP and UDP level (add them to the IPv4 counters).
-**
-** Revision 1.8  2002/07/24 11:13:38  gerlof
-** Changed to ease porting to other UNIX-platforms.
-**
-** Revision 1.7  2002/07/11 09:12:41  root
-** Parsing of /proc/meminfo made 2.5 proof.
-**
-** Revision 1.6  2002/07/10 05:00:21  root
-** Counters pin/pout renamed to swin/swout (Linux conventions).
-**
-** Revision 1.5  2002/07/08 09:31:11  gerlof
-** *** empty log message ***
-**
-** Revision 1.4  2002/07/02 07:36:45  gerlof
-** *** empty log message ***
-**
-** Revision 1.3  2002/07/02 07:08:36  gerlof
-** Recognize more disk driver types via regular expressions
-**
-** Revision 1.2  2002/01/22 13:40:11  gerlof
-** Support for number of cpu's.
-**
-** Revision 1.1  2001/10/02 10:43:31  gerlof
-** Initial revision
-**
 */
-
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
@@ -165,6 +45,7 @@
 #include <dirent.h>
 #include <sys/ioctl.h>
 #include <sys/sysmacros.h>
+#include <limits.h>
 
 #define SCALINGMAXCPU	8	// threshold for scaling info per CPU
 
@@ -183,6 +64,7 @@
 #include <sys/shm.h>
 
 #include "atop.h"
+#include "ifprop.h"
 #include "photosyst.h"
 
 #define	MAXCNT	64
@@ -193,6 +75,12 @@
 #define	MDDTYPE	2
 #define	LVMTYPE	3
 
+/* recognize numa node */
+#define	NUMADIR	"/sys/devices/system/node"
+
+/* Refer to mmzone.h, the default is 11 */
+#define	MAX_ORDER	11
+
 /* hypervisor enum */
 enum {
 	HYPER_NONE	= 0,
@@ -222,15 +110,18 @@ enum {
 
 static int	perfevents = PERF_EVENTS_AUTO;
 static void	getperfevents(struct cpustat *);
+
+static int	run_in_guest(void);
+static int	get_hypervisor(void);
 #endif
 
 static int	get_infiniband(struct ifbstat *);
+static int	get_ksm(struct sstat *);
+static int	get_zswap(struct sstat *);
 
 static int	isdisk(unsigned int, unsigned int,
 			char *, struct perdsk *, int);
 
-static int run_in_guest(void);
-
 static struct ipv6_stats	ipv6_tmp;
 static struct icmpv6_stats	icmpv6_tmp;
 static struct udpv6_stats	udpv6_tmp;
@@ -308,16 +199,104 @@ static struct v6tab 		v6tab[] = {
 
 static int	v6tab_entries = sizeof(v6tab)/sizeof(struct v6tab);
 
+// The following values are used to accumulate cpu statistics per numa.
+// The bitmask realization is from numactl
+#define CPUMASK_SZ (64 * 8)
+
+#define bitsperlong (8 * sizeof(unsigned long))
+#define howmany(x,y) (((x)+((y)-1))/(y))
+#define longsperbits(n) howmany(n, bitsperlong)
+
+#define round_up(x,y) (((x) + (y) - 1) & ~((y)-1))
+#define BITS_PER_LONG (sizeof(unsigned long) * 8)
+#define CPU_BYTES(x) (round_up(x, BITS_PER_LONG)/8)
+#define CPU_LONGS(x) (CPU_BYTES(x) / sizeof(long))
+
+struct bitmask {
+	unsigned long size; /* number of bits in the map */
+	unsigned long long *maskp;
+};
+
+/*
+ * Allocate a bitmask for cpus, of a size large enough to
+ * match the kernel's cpumask_t.
+ */
+struct bitmask *
+numa_allocate_cpumask()
+{
+	int ncpus = CPUMASK_SZ;
+	struct bitmask *bmp;
+
+	bmp = malloc(sizeof(*bmp));
+	ptrverify(bmp, "Malloc failed for numa bitmask");
+
+	bmp->size = ncpus;
+	bmp->maskp = calloc(longsperbits(ncpus), sizeof(unsigned long));
+	ptrverify((bmp->maskp), "Malloc failed for numa maskp");
+
+	return bmp;
+}
+
+void
+numa_bitmask_free(struct bitmask *bmp)
+{
+	if (bmp == 0)
+		return;
+	free(bmp->maskp);
+	bmp->maskp = (unsigned long long *)0xdeadcdef;  /* double free tripwire */
+	free(bmp);
+	return;
+}
+
+int
+numa_parse_bitmap_v2(char *line, struct bitmask *mask)
+{
+	int i;
+	char *p = strchr(line, '\n');
+	if (!p)
+		return -1;
+	int ncpus = mask->size;
+
+	for (i = 0; p > line;i++) {
+		char *oldp, *endp;
+		oldp = p;
+		if (*p == ',')
+			--p;
+		while (p > line && *p != ',')
+			--p;
+		/* Eat two 32bit fields at a time to get longs */
+		if (p > line && sizeof(unsigned long) == 8) {
+			oldp--;
+			memmove(p, p+1, oldp-p+1);
+			while (p > line && *p != ',')
+				--p;
+		}
+		if (*p == ',')
+			p++;
+		if (i >= CPU_LONGS(ncpus))
+			return -1;
+		mask->maskp[i] = strtoul(p, &endp, 16);
+		if (endp != oldp)
+			return -1;
+		p--;
+	}
+	return 0;
+}
+
 void
 photosyst(struct sstat *si)
 {
 	static char	part_stats = 1; /* per-partition statistics ? */
 	static char	ib_stats = 1; 	/* InfiniBand statistics ? */
+	static char	ksm_stats = 1; 	
+	static char	zswap_stats = 1;
 
-	register int	i, nr;
+	register int	i, nr, j;
 	count_t		cnts[MAXCNT];
 	float		lavg1, lavg5, lavg15;
 	FILE 		*fp;
+	DIR		*dirp;
+	struct dirent	*dentry;
 	char		linebuf[1024], nam[64], origdir[1024];
 	unsigned int	major, minor;
 	struct shm_info	shminfo;
@@ -481,7 +460,7 @@ photosyst(struct sstat *si)
             {
                 long long f=0;
 
-                sprintf(fn,
+                snprintf(fn, sizeof fn,
                    "/sys/devices/system/cpu/cpu%d/cpufreq/stats/time_in_state",
                    i);
 
@@ -511,7 +490,7 @@ photosyst(struct sstat *si)
                 }
 		else
 		{    // governor statistics not available
-                     sprintf(fn,  
+                     snprintf(fn, sizeof fn, 
                       "/sys/devices/system/cpu/cpu%d/cpufreq/cpuinfo_max_freq",
 		      i);
 
@@ -530,7 +509,7 @@ photosyst(struct sstat *si)
 	                        si->cpu.cpu[i].freqcnt.maxfreq=0;
                         }
 
-                       sprintf(fn,  
+                       snprintf(fn, sizeof fn,
                        "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_cur_freq",
 		       i);
         
@@ -591,6 +570,12 @@ photosyst(struct sstat *si)
 	** gather virtual memory statistics from the file /proc/vmstat and
 	** store them in binary form (>= kernel 2.6)
 	*/
+	si->mem.oomkills     = -1;
+
+	si->mem.allocstall   = 0;
+	si->mem.numamigrate  = 0;
+	si->mem.pgmigrate    = 0;
+
 	if ( (fp = fopen("vmstat", "r")) != NULL)
 	{
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
@@ -624,9 +609,31 @@ photosyst(struct sstat *si)
 				continue;
 			}
 
-			if ( strcmp("allocstall", nam) == EQ)
+			// more counters might start with "allocstall"
+			if ( memcmp("allocstall", nam, 10) == EQ)
 			{
-				si->mem.allocstall = cnts[0];
+				si->mem.allocstall += cnts[0];
+				continue;
+			}
+
+			if ( strcmp("oom_kill", nam) == EQ)
+			{
+				si->mem.oomkills = cnts[0];
+				continue;
+			}
+
+			if ( strcmp("compact_stall", nam) == EQ) {
+				si->mem.compactstall = cnts[0];
+				continue;
+			}
+
+			if ( strcmp("numa_pages_migrated", nam) == EQ) {
+				si->mem.numamigrate = cnts[0];
+				continue;
+			}
+
+			if ( strcmp("pgmigrate_success", nam) == EQ) {
+				si->mem.pgmigrate = cnts[0];
 				continue;
 			}
 		}
@@ -785,7 +792,7 @@ photosyst(struct sstat *si)
 	** gather vmware-related statistics from /proc/vmmemctl
 	** (only present if balloon driver enabled)
 	*/ 
-	si->mem.vmwballoon = (count_t) 0;
+	si->mem.vmwballoon = (count_t) -1;
 
 	if ( (fp = fopen("vmmemctl", "r")) != NULL)
 	{
@@ -808,7 +815,7 @@ photosyst(struct sstat *si)
 	** searching for:
 	** 	size       4    519101312
 	*/ 
-	si->mem.zfsarcsize = (count_t) 0;
+	si->mem.zfsarcsize = (count_t) -1;
 
 	if ( (fp = fopen("spl/kstat/zfs/arcstats", "r")) != NULL)
 	{
@@ -830,6 +837,195 @@ photosyst(struct sstat *si)
 		fclose(fp);
 	}
 
+	/*
+	** gather per numa memory-related statistics from the file
+	** /sys/devices/system/node/node0/meminfo, and store them in binary form.
+	*/
+	dirp = opendir(NUMADIR);
+
+	if (dirp)
+	{
+		/*
+		** read every directory-entry and search for all numa nodes
+		*/
+		while ( (dentry = readdir(dirp)) )
+		{
+			if (strncmp(dentry->d_name, "node", 4))
+				continue;
+
+			j = strtoul(dentry->d_name + 4, NULL, 0);
+
+			if (j >= MAXNUMA)	// too many NUMA nodes?
+				continue;	// skip (no break, because order unknown)
+
+			si->memnuma.nrnuma++;
+
+			snprintf(fn, sizeof fn, NUMADIR "/%s/meminfo", dentry->d_name);
+
+			if ( (fp = fopen(fn, "r")) != NULL)
+			{
+				while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+				{
+					nr = sscanf(&linebuf[5],
+						"%lld %s %lld\n",
+						&cnts[0], nam, &cnts[1]);
+
+					if (cnts[0] != j)
+						continue;
+
+					if ( strcmp("MemTotal:", nam) == EQ)
+						si->memnuma.numa[j].totmem = cnts[1]*1024/pagesize;
+					else if ( strcmp("MemFree:", nam) == EQ)
+						si->memnuma.numa[j].freemem = cnts[1]*1024/pagesize;
+					else if ( strcmp("FilePages:", nam) == EQ)
+						si->memnuma.numa[j].filepage = cnts[1]*1024/pagesize;
+					else if ( strcmp("Active:", nam) == EQ)
+						si->memnuma.numa[j].active = cnts[1]*1024/pagesize;
+					else if ( strcmp("Inactive:", nam) == EQ)
+						si->memnuma.numa[j].inactive = cnts[1]*1024/pagesize;
+					else if ( strcmp("Dirty:", nam) == EQ)
+						si->memnuma.numa[j].dirtymem = cnts[1]*1024/pagesize;
+					else if ( strcmp("Shmem:", nam) == EQ)
+						si->memnuma.numa[j].shmem = cnts[1]*1024/pagesize;
+					else if ( strcmp("Slab:", nam) == EQ)
+						si->memnuma.numa[j].slabmem = cnts[1]*1024/pagesize;
+					else if ( strcmp("SReclaimable:", nam) == EQ)
+						si->memnuma.numa[j].slabreclaim = cnts[1]*1024/pagesize;
+					else if ( strcmp("HugePages_Total:", nam) == EQ)
+						si->memnuma.numa[j].tothp = cnts[1];
+				}
+				fclose(fp);
+			}
+		}
+		closedir(dirp);
+	}
+
+	/* gather fragmentation level for per numa, only for 'Normal' */
+	if (si->memnuma.nrnuma > 0)
+	{
+		char tmp[64];
+		float frag[MAX_ORDER];
+
+		/* If kernel CONFIG_COMPACTION is enabled, get the percentage directly */
+		if ( (fp = fopen("/sys/kernel/debug/extfrag/unusable_index", "r")) != NULL )
+		{
+			while ( fgets(linebuf, sizeof(linebuf), fp) != NULL )
+			{
+				nr = sscanf(&linebuf[5],
+					"%lld, %s %s %f %f %f %f %f %f %f %f %f %f %f\n",
+					&cnts[0], tmp, nam, &frag[0], &frag[1], &frag[2],
+					&frag[3], &frag[4], &frag[5], &frag[6], &frag[7],
+					&frag[8], &frag[9], &frag[10]);
+
+				if ( nr < 3 || strcmp("Normal", nam) != 0 )
+					continue;
+
+				for (i = 0; i < MAX_ORDER; i++)
+					si->memnuma.numa[cnts[0]].frag += frag[i];
+
+				si->memnuma.numa[cnts[0]].frag /= MAX_ORDER;
+			}
+			fclose(fp);
+		}
+		/* If CONFIG_COMPACTION is not enabled, calculate from buddyinfo file */
+		else if ( (fp = fopen("/proc/buddyinfo", "r")) != NULL )
+		{
+			count_t free_page[MAX_ORDER];
+			count_t total_free, prev_free;
+			float total_frag;
+
+			while ( fgets(linebuf, sizeof(linebuf), fp) != NULL )
+			{
+				nr = sscanf(&linebuf[5],
+					"%lld, %s %s %lld %lld %lld %lld %lld "
+					"%lld %lld %lld %lld %lld %lld\n",
+					&cnts[0], tmp, nam, &cnts[1], &cnts[2], &cnts[3],
+					&cnts[4], &cnts[5], &cnts[6], &cnts[7], &cnts[8],
+					&cnts[9], &cnts[10], &cnts[11]);
+
+				if (nr < 3 || strcmp("Normal", nam) != 0 )
+					continue;
+
+				/* get free pages numbers for each order, and total free pages */
+				total_free = 0;
+				total_frag = 0.00;
+				for (i = 0; i < MAX_ORDER; i++)
+				{
+					free_page[i] = cnts[i+1] << i;
+					total_free += free_page[i];
+				}
+				/* get fragmentation level for each order, then summarize */
+				for (i = 0; i < MAX_ORDER; i++)
+				{
+					prev_free = 0;
+					for (j = 0; j < i; j++)
+						prev_free += free_page[j];
+					total_frag += (float)prev_free/total_free;
+				}
+
+				if (cnts[0] < MAXNUMA)
+					si->memnuma.numa[cnts[0]].frag = total_frag/MAX_ORDER;
+			}
+			fclose(fp);
+		}
+	}
+
+	/*
+	** accumulate each cpu statistic for per NUMA, and identify numa/cpu
+	** relationship from /sys/devices/system/node/node0/cpumap.
+	*/
+	if (si->memnuma.nrnuma > 1)
+	{
+		char *line = NULL;
+		size_t len = 0;
+		struct bitmask *mask;
+
+		mask = numa_allocate_cpumask();
+
+		si->cpunuma.nrnuma = si->memnuma.nrnuma;
+
+		for (j=0; j < si->cpunuma.nrnuma; j++)
+		{
+			snprintf(fn, sizeof fn, NUMADIR "/node%d/cpumap", j);
+
+			if ( (fp = fopen(fn, "r")) != 0)
+			{
+				if ( getdelim(&line, &len, '\n', fp) > 0 )
+				{
+					if (numa_parse_bitmap_v2(line, mask) < 0)
+					{
+						mcleanstop(54, "failed to parse numa bitmap\n");
+					}
+				}
+				fclose(fp);
+			}
+
+			for (i=0; i < mask->size; i++)
+			{
+				if ( (mask->maskp[i/bitsperlong] >> (i % bitsperlong)) & 1 )
+				{
+					si->cpunuma.numa[j].nrcpu++;
+					si->cpunuma.numa[j].utime += si->cpu.cpu[i].utime;
+					si->cpunuma.numa[j].ntime += si->cpu.cpu[i].ntime;
+					si->cpunuma.numa[j].stime += si->cpu.cpu[i].stime;
+					si->cpunuma.numa[j].itime += si->cpu.cpu[i].itime;
+					si->cpunuma.numa[j].wtime += si->cpu.cpu[i].wtime;
+					si->cpunuma.numa[j].Itime += si->cpu.cpu[i].Itime;
+					si->cpunuma.numa[j].Stime += si->cpu.cpu[i].Stime;
+					si->cpunuma.numa[j].steal += si->cpu.cpu[i].steal;
+					si->cpunuma.numa[j].guest += si->cpu.cpu[i].guest;
+				}
+			}
+		}
+
+		free(line);
+		numa_bitmask_free(mask);
+	}
+	else
+	{
+		si->cpunuma.nrnuma = 0;
+	}
+
 	/*
 	** gather network-related statistics
  	** 	- interface stats from the file /proc/net/dev
@@ -840,8 +1036,11 @@ photosyst(struct sstat *si)
 	/*
 	** interface statistics
 	*/
+	initifprop();   // periodically refresh interface properties
+
 	if ( (fp = fopen("net/dev", "r")) != NULL)
 	{
+		struct ifprop ifprop;
 		char *cp;
 
 		i = 0;
@@ -873,11 +1072,30 @@ photosyst(struct sstat *si)
 				&(si->intf.intf[i].scarrier),
 				&(si->intf.intf[i].scompr));
 
-			if (nr == 17)	/* skip header & lines without stats */
-			{
-				if (++i >= MAXINTF-1)
-					break;
-			}
+			/*
+			** skip header line and lines without stats
+			*/
+			if (nr != 17)
+				continue;
+
+			/*
+			** skip interfaces that are invalidated
+			** (mainly virtual interfaces)
+			** because the total number of interfaces
+			** exceeds the maximum supported by atop (MAXINTF)
+			*/
+			strcpy(ifprop.name, si->intf.intf[i].name);
+
+			if (!getifprop(&ifprop))
+				continue;
+
+			/*
+			** accept this interface but skip the remaining
+			** interfaces because we reached the total number
+			** of interfaces supported by atop (MAXINTF)
+			*/
+			if (++i >= MAXINTF-1)
+				break;
 		}
 
 		si->intf.intf[i].name[0] = '\0'; /* set terminator for table */
@@ -1056,21 +1274,35 @@ photosyst(struct sstat *si)
 
 		while ( fgets(linebuf, sizeof(linebuf), fp) )
 		{
+			/* discards are not supported in older kernels */
+ 			tmpdsk.ndisc = -1;
+
 			nr = sscanf(linebuf,
-			      "%d %d %255s %lld %*d %lld %*d "
-			      "%lld %*d %lld %*d %*d %lld %lld",
+			      "%d %d %255s "		// ident
+                              "%lld %*d %lld %*d "	// reads
+			      "%lld %*d %lld %*d "	// writes
+			      "%*d %lld %lld "		// misc
+			      "%lld %*d %lld %*d",	// discards
 				&major, &minor, diskname,
 				&tmpdsk.nread,  &tmpdsk.nrsect,
 				&tmpdsk.nwrite, &tmpdsk.nwsect,
-				&tmpdsk.io_ms,  &tmpdsk.avque );
+				&tmpdsk.io_ms,  &tmpdsk.avque,
+				&tmpdsk.ndisc,  &tmpdsk.ndsect);
 
-			/*
-			** check if this line concerns the entire disk
-			** or just one of the partitions of a disk (to be
-			** skipped)
-			*/
-			if (nr == 9)	/* full stats-line ? */
+			if (nr >= 9)	/* full stats-line ? */
 			{
+				/*
+  				** when no transfers issued, skip disk (partition)
+				*/
+				if (tmpdsk.nread + tmpdsk.nwrite +
+				     (tmpdsk.ndisc == -1 ? 0 : tmpdsk.ndisc) == 0)
+					continue;
+
+				/*
+				** check if this line concerns the entire disk
+				** or just one of the partitions of a disk (to be
+				** skipped)
+				*/
 				switch ( isdisk(major, minor, diskname,
 							 &tmpdsk, MAXDKNAM) )
 				{
@@ -1542,6 +1774,18 @@ photosyst(struct sstat *si)
 	if (ib_stats)
 		ib_stats = get_infiniband(&(si->ifb));
 
+	/*
+	** get counters related to ksm
+	*/
+	if (ksm_stats)
+		ksm_stats = get_ksm(si);
+
+	/*
+	** get counters related to zswap
+	*/
+	if (zswap_stats)
+		zswap_stats = get_zswap(si);
+
 	/*
  	** return to original directory
 	*/
@@ -1717,6 +1961,7 @@ static struct {
 	{ "^md[0-9][0-9]*$",			{0},  nullmodname, MDDTYPE, },
 	{ "^vd[a-z][a-z]*$",                    {0},  nullmodname, DSKTYPE, },
 	{ "^nvme[0-9][0-9]*n[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
+	{ "^nvme[0-9][0-9]*c[0-9][0-9]*n[0-9][0-9]*$", {0}, nullmodname, DSKTYPE, },
 	{ "^nbd[0-9][0-9]*$",			{0},  nullmodname, DSKTYPE, },
 	{ "^hd[a-z]$",				{0},  nullmodname, DSKTYPE, },
 	{ "^rd/c[0-9][0-9]*d[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
@@ -2105,6 +2350,222 @@ ibstat(struct ibcachent *ibc, struct perifb *ifb)
 	return 1;
 }
 
+/*
+** retrieve ksm values (if switched on)
+*/
+static int
+get_ksm(struct sstat *si)
+{
+	FILE *fp;
+	int  state;
+
+	si->mem.ksmsharing = -1;
+	si->mem.ksmshared  = -1;
+
+	if ((fp=fopen("/sys/kernel/mm/ksm/run", "r")) != 0)
+	{
+		if (fscanf(fp, "%d", &state) == 1)
+		{
+			if (state == 0)
+			{
+				fclose(fp);
+				return 0; // no more calling
+			}
+		}
+
+		fclose(fp);
+	}
+
+	if ((fp=fopen("/sys/kernel/mm/ksm/pages_sharing", "r")) != 0)
+	{
+		if (fscanf(fp, "%llu", &(si->mem.ksmsharing)) != 1)
+			si->mem.ksmsharing = 0;
+
+		fclose(fp);
+	}
+
+	if ((fp=fopen("/sys/kernel/mm/ksm/pages_shared", "r")) != 0)
+	{
+		if (fscanf(fp, "%llu", &(si->mem.ksmshared)) != 1)
+			si->mem.ksmshared = 0;
+
+		fclose(fp);
+	}
+
+	return 1;
+}
+
+/*
+** retrieve zswap values (if switched on)
+*/
+static int
+get_zswap(struct sstat *si)
+{
+	FILE *fp;
+	char  state;
+
+	si->mem.zswtotpool = -1;
+	si->mem.zswstored  = -1;
+
+	if ((fp=fopen("/sys/module/zswap/parameters/enabled", "r")) != 0)
+	{
+		if (fscanf(fp, "%c", &state) == 1)
+		{
+			if (state != 'Y')
+			{
+				fclose(fp);
+				return 0; // no more calling
+			}
+		}
+
+		fclose(fp);
+	}
+
+	regainrootprivs();
+
+	if ((fp=fopen("/sys/kernel/debug/zswap/pool_total_size", "r")) != 0)
+	{
+		if (fscanf(fp, "%llu", &(si->mem.zswtotpool)) != 1)
+			si->mem.zswtotpool = 0;
+		else
+			si->mem.zswtotpool /= pagesize;
+
+		fclose(fp);
+	}
+
+	if ((fp=fopen("/sys/kernel/debug/zswap/stored_pages", "r")) != 0)
+	{
+		if (fscanf(fp, "%llu", &(si->mem.zswstored)) != 1)
+			si->mem.zswstored = 0;
+
+		fclose(fp);
+	}
+
+	if (! droprootprivs())
+		mcleanstop(42, "failed to drop root privs\n");
+
+	return 1;
+}
+
+
+#if	HTTPSTATS
+/*
+** retrieve statistics from local HTTP daemons
+** via http://localhost/server-status?auto
+*/
+int
+getwwwstat(unsigned short port, struct wwwstat *wp)
+{
+	int 			sockfd, tobefound;
+	FILE			*sockfp;
+	struct sockaddr_in	sockname;
+	char			linebuf[4096];
+	char			label[512];
+	long long		value;
+
+	memset(wp, 0, sizeof *wp);
+
+	/*
+	** allocate a socket and connect to the local HTTP daemon
+	*/
+	if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
+		return 0;
+
+	sockname.sin_family		= AF_INET;
+	sockname.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);
+	sockname.sin_port		= htons(port);
+
+	if ( connect(sockfd, (struct sockaddr *) &sockname,
+						sizeof sockname) == -1)
+	{
+		close(sockfd);
+		return 0;
+	}
+
+	/*
+	** write a GET-request for /server-status
+	*/
+	if ( write(sockfd, HTTPREQ, sizeof HTTPREQ) < sizeof HTTPREQ)
+	{
+		close(sockfd);
+		return 0;
+	}
+
+	/*
+	** remap socket descriptor to a stream to allow stdio calls
+	*/
+	sockfp = fdopen(sockfd, "r+");
+
+	/*
+	** read response line by line
+	*/
+	tobefound = 5;		/* number of values to be searched */
+
+	while ( fgets(linebuf, sizeof linebuf, sockfp) && tobefound)
+	{
+		/*
+		** handle line containing status code
+		*/
+		if ( strncmp(linebuf, "HTTP/", 5) == 0)
+		{
+			sscanf(linebuf, "%511s %lld %*s\n", label, &value);
+
+			if (value != 200)	/* HTTP-request okay? */
+			{
+				fclose(sockfp);
+				close(sockfd);
+				return 0;
+			}
+
+			continue;
+		}
+
+		/*
+		** decode line and search for the required counters
+		*/
+		if (sscanf(linebuf, "%511[^:]: %lld\n", label, &value) == 2)
+		{
+			if ( strcmp(label, "Total Accesses") == 0)
+			{
+				wp->accesses = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "Total kBytes") == 0)
+			{
+				wp->totkbytes = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "Uptime") == 0)
+			{
+				wp->uptime = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "BusyWorkers") == 0)
+			{
+				wp->bworkers = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "IdleWorkers") == 0)
+			{
+				wp->iworkers = value;
+				tobefound--;
+			}
+		}
+	}
+
+	fclose(sockfp);
+	close(sockfd);
+
+	return 1;
+}
+#endif
+
+
+
 /*
 ** retrieve low-level CPU events:
 ** 	instructions and cycles per CPU
@@ -2261,130 +2722,12 @@ getperfevents(struct cpustat *cs)
 		}
         }
 }
-#else
-void
-do_perfevents(char *tagname, char *tagvalue)
+
+static int
+run_in_guest(void)
 {
-	wcleanstop(1, "atop built with NOPERFEVENT, cannot use perfevents\n");
+	return get_hypervisor() != HYPER_NONE;
 }
-#endif
-
-#if	HTTPSTATS
-/*
-** retrieve statistics from local HTTP daemons
-** via http://localhost/server-status?auto
-*/
-int
-getwwwstat(unsigned short port, struct wwwstat *wp)
-{
-	int 			sockfd, tobefound;
-	FILE			*sockfp;
-	struct sockaddr_in	sockname;
-	char			linebuf[4096];
-	char			label[512];
-	long long		value;
-
-	memset(wp, 0, sizeof *wp);
-
-	/*
-	** allocate a socket and connect to the local HTTP daemon
-	*/
-	if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-		return 0;
-
-	sockname.sin_family		= AF_INET;
-	sockname.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);
-	sockname.sin_port		= htons(port);
-
-	if ( connect(sockfd, (struct sockaddr *) &sockname,
-						sizeof sockname) == -1)
-	{
-		close(sockfd);
-		return 0;
-	}
-
-	/*
-	** write a GET-request for /server-status
-	*/
-	if ( write(sockfd, HTTPREQ, sizeof HTTPREQ) < sizeof HTTPREQ)
-	{
-		close(sockfd);
-		return 0;
-	}
-
-	/*
-	** remap socket descriptor to a stream to allow stdio calls
-	*/
-	sockfp = fdopen(sockfd, "r+");
-
-	/*
-	** read response line by line
-	*/
-	tobefound = 5;		/* number of values to be searched */
-
-	while ( fgets(linebuf, sizeof linebuf, sockfp) && tobefound)
-	{
-		/*
-		** handle line containing status code
-		*/
-		if ( strncmp(linebuf, "HTTP/", 5) == 0)
-		{
-			sscanf(linebuf, "%511s %lld %*s\n", label, &value);
-
-			if (value != 200)	/* HTTP-request okay? */
-			{
-				fclose(sockfp);
-				close(sockfd);
-				return 0;
-			}
-
-			continue;
-		}
-
-		/*
-		** decode line and search for the required counters
-		*/
-		if (sscanf(linebuf, "%511[^:]: %lld\n", label, &value) == 2)
-		{
-			if ( strcmp(label, "Total Accesses") == 0)
-			{
-				wp->accesses = value;
-				tobefound--;
-			}
-
-			if ( strcmp(label, "Total kBytes") == 0)
-			{
-				wp->totkbytes = value;
-				tobefound--;
-			}
-
-			if ( strcmp(label, "Uptime") == 0)
-			{
-				wp->uptime = value;
-				tobefound--;
-			}
-
-			if ( strcmp(label, "BusyWorkers") == 0)
-			{
-				wp->bworkers = value;
-				tobefound--;
-			}
-
-			if ( strcmp(label, "IdleWorkers") == 0)
-			{
-				wp->iworkers = value;
-				tobefound--;
-			}
-		}
-	}
-
-	fclose(sockfp);
-	close(sockfd);
-
-	return 1;
-}
-#endif
-
 
 #if defined(__x86_64__) || defined(__i386__)
 #define HYPERVISOR_INFO_LEAF   0x40000000
@@ -2447,8 +2790,11 @@ get_hypervisor(void)
 }
 #endif
 
-static int
-run_in_guest(void)
+#else /* ! NOPERFEVENT */
+void
+do_perfevents(char *tagname, char *tagvalue)
 {
-	return get_hypervisor() != HYPER_NONE;
+	if (strcmp("disable", tagvalue))
+		mcleanstop(1, "atop built with NOPERFEVENT, cannot use perfevents\n");
 }
+#endif
