@@ -28,6 +28,7 @@
 
 #define	MAXCPU		2048
 #define	MAXDSK		1024
+#define	MAXNUMA		1024
 #define	MAXLVM		2048
 #define	MAXMDD		256
 #define	MAXINTF		128
@@ -75,7 +76,56 @@ struct	memstat {
 	count_t	vmwballoon;	// vmware claimed balloon pages
 	count_t	zfsarcsize;	// zfsonlinux ARC size (pages)
 	count_t swapcached;	// swap cache (pages)
-	count_t	cfuture[6];	// reserved for future use
+	count_t	ksmsharing;	// saved i.e. deduped memory (pages)
+	count_t	ksmshared;	// current size shared pages (pages)
+	count_t	zswstored;	// zswap stored pages (pages)
+	count_t	zswtotpool;	// total pool size (pages)
+	count_t	oomkills;	// number of oom killings
+	count_t	compactstall;	// counter for process stalls
+	count_t	pgmigrate;	// counter for migrated successfully (pages)
+	count_t	numamigrate;	// counter for numa migrated (pages)
+	count_t	cfuture[9];	// reserved for future use
+};
+
+/************************************************************************/
+
+struct	mempernuma {
+	float	frag;		// fragmentation level for this numa
+	count_t	totmem;		// number of physical pages for this numa
+	count_t	freemem;	// number of free     pages for this numa
+	count_t	filepage;	// number of file     pages for this numa
+	count_t	dirtymem;	// number of cache    pages (dirty) for this numa
+	count_t	slabmem;	// number of slab     pages for this numa
+	count_t	slabreclaim;	// reclaimable slab (pages) for this numa
+
+	count_t	active;		// number of pages used more recently for this numa
+	count_t	inactive;	// number of pages less recently used for this numa
+
+	count_t	shmem;		// tot shmem incl. tmpfs (pages) for this numa
+	count_t	tothp;		// total huge pages (huge pages) for this numa
+};
+
+struct	memnuma {
+	count_t           nrnuma;		/* the counts of numa		*/
+	struct mempernuma numa[MAXNUMA];
+};
+
+struct	cpupernuma {
+	count_t	nrcpu;		// number of cpu's
+	count_t	stime;		// accumulate system  time in clock ticks for per numa
+	count_t	utime;		// accumulate user    time in clock ticks for per numa
+	count_t	ntime;		// accumulate nice    time in clock ticks for per numa
+	count_t	itime;		// accumulate idle    time in clock ticks for per numa
+	count_t	wtime;		// accumulate iowait  time in clock ticks for per numa
+	count_t	Itime;		// accumulate irq     time in clock ticks for per numa
+	count_t	Stime;		// accumulate softirq time in clock ticks for per numa
+	count_t	steal;		// accumulate steal   time in clock ticks for per numa
+	count_t	guest;		// accumulate guest   time in clock ticks for per numa
+};
+
+struct	cpunuma {
+	count_t           nrnuma;		/* the counts of numa		*/
+	struct cpupernuma numa[MAXNUMA];
 };
 
 /************************************************************************/
@@ -115,7 +165,7 @@ struct percpu {
         struct freqcnt	freqcnt;/* frequency scaling info  		*/
 	count_t		instr;	/* CPU instructions 			*/
 	count_t		cycle;	/* CPU cycles 				*/
-	count_t		cfuture[2];	/* reserved for future use	*/
+	count_t		cfuture[6];	/* reserved for future use	*/
 };
 
 struct	cpustat {
@@ -142,7 +192,9 @@ struct	perdsk {
         count_t	nwsect;	/* number of sectors written            */
         count_t	io_ms;	/* number of millisecs spent for I/O    */
         count_t	avque;	/* average queue length                 */
-	count_t	cfuture[4];	/* reserved for future use	*/
+        count_t	ndisc;	/* number of discards (-1 = unavailable)*/
+        count_t	ndsect;	/* number of sectors discarded          */
+	count_t	cfuture[2];	/* reserved for future use	*/
 };
 
 struct dskstat {
@@ -179,7 +231,7 @@ struct	perintf {
 	count_t scompr; /* transmit compressed                  */
 	count_t	sfuture[4];	/* reserved for future use	*/
 
-	char 	type;	/* interface type ('e'/'w'/'?')  	*/
+	char 	type;	/* interface type ('e'/'w'/'v'/'?')  	*/
 	long 	speed;	/* interface speed in megabits/second	*/
 	long 	speedp;	/* previous interface speed 		*/
 	char	duplex;	/* full duplex (boolean) 		*/
@@ -355,6 +407,8 @@ struct	sstat {
 	struct memstat	mem;
 	struct netstat	net;
 	struct intfstat	intf;
+	struct memnuma	memnuma;
+	struct cpunuma	cpunuma;
 	struct dskstat  dsk;
 	struct nfsstat  nfs;
 	struct contstat cfs;
