@@ -54,6 +54,7 @@
 #include "rawlog.h"
 
 #define	BASEPATH	"/var/log/atop"  
+#define	BINPATH		"/usr/bin/atop"
 
 static int	getrawrec  (int, struct rawrecord *, int);
 static int	getrawsstat(int, struct sstat *, int);
@@ -989,7 +990,7 @@ readchunk(int fd, void *buf, int len)
 static void
 try_other_version(int majorversion, int minorversion)
 {
-	char		tmpbuf[1024], *p;
+	char		tmpbuf[1024];
 	extern char	**argvp;
 	int		fds;
 	struct rlimit	rlimit;
@@ -1000,8 +1001,7 @@ try_other_version(int majorversion, int minorversion)
 	** the current pathname (if any) is stripped off
 	*/
 	snprintf(tmpbuf, sizeof tmpbuf, "%s-%d.%d",
-		(p = strrchr(*argvp, '/')) ? p+1 : *argvp,
-			majorversion, minorversion);
+		BINPATH, majorversion, minorversion);
 
 	fprintf(stderr, "trying to activate %s....\n", tmpbuf);
 
