@@ -28,231 +28,6 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
-** 
-** $Log: showgeneric.c,v $
-** Revision 1.71  2010/10/25 19:08:32  gerlof
-** When the number of lines is too small for the system-level
-** lines, limit the number of variable resources automatically
-** to a minimum.
-**
-** Revision 1.70  2010/10/23 14:04:05  gerlof
-** Counters for total number of running and sleep threads (JC van Winkel).
-**
-** Revision 1.69  2010/04/23 12:19:35  gerlof
-** Modified mail-address in header.
-**
-** Revision 1.68  2010/04/23 09:58:11  gerlof
-** Version (flag -V) handled earlier after startup.
-**
-** Revision 1.67  2010/04/23 07:57:32  gerlof
-** Proper sorting of processes when switching from single process view
-** to cumulative view (key 'u' or 'p') and vice versa.
-**
-** Revision 1.66  2010/04/17 17:20:26  gerlof
-** Allow modifying the layout of the columns in the system lines.
-**
-** Revision 1.65  2010/03/16 21:13:38  gerlof
-** Program and user selection can be combined with program and user
-** accumulation.
-**
-** Revision 1.64  2010/03/16 20:18:46  gerlof
-** Show in header-line if user selections and program selection are active.
-**
-** Revision 1.63  2010/03/16 20:08:51  gerlof
-** Performance improvement: only sort system-resources once per interval.
-**
-** Revision 1.62  2010/03/04 10:53:01  gerlof
-** Support I/O-statistics on logical volumes and MD devices.
-**
-** Revision 1.61  2009/12/17 10:55:07  gerlof
-** *** empty log message ***
-**
-** Revision 1.60  2009/12/17 10:50:30  gerlof
-** Allow own defined process line with key 'o' and a definition
-** in the atoprc file.
-**
-** Revision 1.59  2009/12/17 09:03:26  gerlof
-** Center message "....since boot" in status line on first screen.
-**
-** Revision 1.58  2009/12/17 08:55:15  gerlof
-** Show messages on status line in color to draw attention.
-**
-** Revision 1.57  2009/12/17 08:16:14  gerlof
-** Introduce branch-key to go to specific time in raw file.
-**
-** Revision 1.56  2009/12/12 09:06:39  gerlof
-** Corrected cumulated disk I/O per user/program (JC van Winkel).
-**
-** Revision 1.55  2009/12/10 13:34:44  gerlof
-** Show which toggle-keys are active in the header line.
-**
-** Revision 1.54  2009/12/10 11:55:03  gerlof
-** Introduce system-wide /etc/atoprc
-**
-** Revision 1.53  2009/12/10 09:53:08  gerlof
-** Improved display of header-line (JC van Winkel).
-**
-** Revision 1.52  2008/03/06 10:14:01  gerlof
-** Modified help-messages.
-**
-** Revision 1.51  2008/02/25 13:47:21  gerlof
-** Bug-solution: segmentation-fault in case of invalid regular expression.
-**
-** Revision 1.50  2008/01/07 11:33:58  gerlof
-** Cosmetic changes.
-**
-** Revision 1.49  2008/01/07 10:18:24  gerlof
-** Implement possibility to make summaries with atopsar.
-**
-** Revision 1.48  2007/03/22 10:12:17  gerlof
-** Support for io counters (>= kernel 2.6.20).
-**
-** Revision 1.47  2007/03/21 14:22:34  gerlof
-** Handle io counters maintained from 2.6.20
-**
-** Revision 1.46  2007/03/20 11:13:15  gerlof
-** Cosmetic changes.
-**
-** Revision 1.45  2007/03/09 12:39:59  gerlof
-** Do not allow 'N' and 'D' when kernel-patch is not installed.
-**
-** Revision 1.44  2007/02/13 10:31:53  gerlof
-** Removal of external declarations.
-**
-** Revision 1.43  2007/01/18 10:41:45  gerlof
-** Add support for colors.
-** Add support for automatic determination of most critical resource.
-**
-** Revision 1.42  2006/11/13 13:48:36  gerlof
-** Implement load-average counters, context-switches and interrupts.
-**
-** Revision 1.41  2006/04/03 05:42:35  gerlof
-** *** empty log message ***
-**
-** Revision 1.40  2006/02/07 08:28:26  gerlof
-** Improve screen-handling (less flashing) by exchanging clear()
-** by werase() (contribution Folkert van Heusden).
-**
-** Revision 1.39  2005/11/04 14:16:16  gerlof
-** Minor bug-solutions.
-**
-** Revision 1.38  2005/10/28 09:52:03  gerlof
-** All flags/subcommands are defined as macro's.
-** Subcommand 'p' has been changed to 'z' (pause).
-**
-** Revision 1.37  2005/10/24 06:12:17  gerlof
-** Flag -L modified into -l.
-**
-** Revision 1.36  2005/10/21 09:50:46  gerlof
-** Per-user accumulation of resource consumption.
-** Possibility to send signal to process.
-**
-** Revision 1.35  2004/12/14 15:06:41  gerlof
-** Implementation of patch-recognition for disk and network-statistics.
-**
-** Revision 1.34  2004/09/27 11:01:13  gerlof
-** Corrected usage-info as suggested by Edelhard Becker.
-**
-** Revision 1.33  2004/09/13 09:19:14  gerlof
-** Modify subcommands (former 's' -> 'v', 'v' -> 'V', new 's').
-**
-** Revision 1.32  2004/08/31 09:52:47  root
-** information about underlying threads.
-**
-** Revision 1.31  2004/06/01 11:57:58  gerlof
-** Regular expressions for selections on process-name and user-name.
-**
-** Revision 1.30  2003/07/07 09:27:24  gerlof
-** Cleanup code (-Wall proof).
-**
-** Revision 1.29  2003/07/03 11:16:42  gerlof
-** Implemented subcommand `r' (reset).
-**
-** Revision 1.28  2003/06/30 11:29:49  gerlof
-** Handle configuration file ~/.atoprc
-**
-** Revision 1.27  2003/06/24 06:21:57  gerlof
-** Limit number of system resource lines.
-**
-** Revision 1.26  2003/02/07 10:19:18  gerlof
-** Possibility to show the version number and date.
-**
-** Revision 1.25  2003/01/17 07:32:16  gerlof
-** Show the full command-line per process (option 'c').
-**
-** Revision 1.24  2002/10/30 13:47:20  gerlof
-** Generate notification for statistics since boot.
-**
-** Revision 1.23  2002/10/08 12:00:30  gerlof
-** *** empty log message ***
-**
-** Revision 1.22  2002/09/26 14:17:39  gerlof
-** No beep when resizing the window.
-**
-** Revision 1.21  2002/09/26 13:52:26  gerlof
-** Limit header lines by not showing disks.
-**
-** Revision 1.20  2002/09/18 07:15:59  gerlof
-** Modified viewflag to rawreadflag.
-**
-** Revision 1.19  2002/09/17 13:17:39  gerlof
-** Allow key 'T' to be pressed to view previous sample in raw file.
-**
-** Revision 1.18  2002/08/30 07:11:50  gerlof
-** Minor changes to support viewing of raw atop data.
-**
-** Revision 1.17  2002/08/27 12:10:12  gerlof
-** Allow raw data file to be written and to be read (with compression).
-**
-** Revision 1.16  2002/07/24 11:12:46  gerlof
-** Changed to ease porting to other UNIX-platforms.
-**
-** Revision 1.15  2002/07/11 09:12:05  root
-** Some minor updates.
-**
-** Revision 1.14  2002/07/10 05:00:37  root
-** Counters pin/pout renamed to swin/swout (Linux conventions).
-**
-** Revision 1.13  2002/07/08 09:29:49  root
-** Limitation for username and groupname (8 characters truncate).
-**
-** Revision 1.12  2002/07/02 07:14:02  gerlof
-** More positions for the name of the disk-unit in the DSK-line.
-**
-** Revision 1.11  2002/01/22 13:40:42  gerlof
-** Support for number of cpu's.
-** Check if the window is large enough for the system-statistics.
-**
-** Revision 1.10  2001/11/30 09:09:36  gerlof
-** Cosmetic chnage.
-**
-** Revision 1.9  2001/11/29 10:41:44  gerlof
-** *** empty log message ***
-**
-** Revision 1.8  2001/11/29 10:38:16  gerlof
-** Exit-code correctly printed.
-**
-** Revision 1.7  2001/11/26 11:18:45  gerlof
-** Modified generic output in case that the kernel-patch is not installed.
-**
-** Revision 1.6  2001/11/13 08:24:50  gerlof
-** Show blank columns for sockets and disk I/O when no kernel-patch installed.
-**
-** Revision 1.5  2001/11/07 09:19:28  gerlof
-** Use /proc instead of /dev/kmem for process-level statistics.
-**
-** Revision 1.4  2001/10/05 13:46:32  gerlof
-** Implemented paging through the process-list
-**
-** Revision 1.3  2001/10/04 08:47:27  gerlof
-** Improved handling of error-messages
-**
-** Revision 1.2  2001/10/03 08:57:53  gerlof
-** Improved help-screen shown in scrollable window
-**
-** Revision 1.1  2001/10/02 10:43:34  gerlof
-** Initial revision
-**
 */
 
 #include <sys/types.h>
@@ -260,19 +35,20 @@
 #include <sys/stat.h>
 #include <sys/utsname.h>
 #include <signal.h>
+#include <ctype.h>
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
-#include <unistd.h>
 #include <stdarg.h>
 #include <curses.h>
 #include <pwd.h>
 #include <grp.h>
 #include <regex.h>
 #include <locale.h>
+#include <unistd.h>
 
 #include "atop.h"
 #include "photoproc.h"
@@ -280,8 +56,10 @@
 #include "showgeneric.h"
 #include "showlinux.h"
 
+int getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);
+
 static struct pselection procsel = {"", {USERSTUB, }, {0,},
-                                    "", 0, { 0, },  "", 0, { 0, } };
+                                    "", 0, { 0, },  "", 0, { 0, }, "", "" };
 static struct sselection syssel;
 
 static void	showhelp(int);
@@ -304,6 +82,7 @@ static int	maxintlines = 999;  /* maximum interface lines          */
 static int	maxifblines = 999;  /* maximum infinibnd lines          */
 static int	maxnfslines = 999;  /* maximum nfs mount lines          */
 static int	maxcontlines = 999; /* maximum container lines          */
+static int	maxnumalines = 999; /* maximum numa      lines          */
 
 static short	colorinfo   = COLOR_GREEN;
 static short	coloralmost = COLOR_CYAN;
@@ -477,6 +256,14 @@ generic_samp(time_t curtime, int nsecs,
 		if (sstat->cfs.nrcontainer > 1 && maxcontlines > 0)
 			qsort(sstat->cfs.cont, sstat->cfs.nrcontainer,
 		  	       sizeof sstat->cfs.cont[0], contcompar);
+
+		if (sstat->memnuma.nrnuma > 1 && maxnumalines > 0)
+			qsort(sstat->memnuma.numa, sstat->memnuma.nrnuma,
+			       sizeof sstat->memnuma.numa[0], memnumacompar);
+
+		if (sstat->cpunuma.nrnuma > 1 && maxnumalines > 0)
+			qsort(sstat->cpunuma.numa, sstat->cpunuma.nrnuma,
+			       sizeof sstat->cpunuma.numa[0], cpunumacompar);
 	}
 
 	/*
@@ -507,11 +294,11 @@ generic_samp(time_t curtime, int nsecs,
 
                 int seclen	= val2elapstr(nsecs, buf);
                 int lenavail 	= (screen ? COLS : linelen) -
-						51 - seclen - utsnodenamelen;
+						52 - seclen - utsnodenamelen;
                 int len1	= lenavail / 3;
                 int len2	= lenavail - len1 - len1; 
 
-		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%"
+		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%"
 		       "*s%s elapsed", 
 			utsname.nodename, len1, "", 
 			format1, format2, len1, "",
@@ -530,6 +317,7 @@ generic_samp(time_t curtime, int nsecs,
 			procsel.container[0]	      ? MSELCONT   : '-',
 			procsel.pid[0] != 0	      ? MSELPID    : '-',
 			procsel.argnamesz	      ? MSELARG    : '-',
+			procsel.states[0]	      ? MSELSTATE  : '-',
 			syssel.lvmnamesz +
 			syssel.dsknamesz +
 			syssel.itfnamesz	      ? MSELSYS    : '-',
@@ -569,7 +357,7 @@ generic_samp(time_t curtime, int nsecs,
 		                  maxcpulines, maxgpulines, maxdsklines,
 				  maxmddlines, maxlvmlines,
 		                  maxintlines, maxifblines, maxnfslines,
-		                  maxcontlines);
+		                  maxcontlines, maxnumalines);
 
 		/*
  		** if system-wide statistics do not fit,
@@ -592,7 +380,7 @@ generic_samp(time_t curtime, int nsecs,
 					maxdsklines, maxmddlines,
 		                        maxlvmlines, maxintlines,
 					maxifblines, maxnfslines,
-		                        maxcontlines);
+		                        maxcontlines, maxnumalines);
 
 			/*
  			** if system-wide statistics still do not fit,
@@ -644,6 +432,29 @@ generic_samp(time_t curtime, int nsecs,
 		{
 			if (flag&RRBOOT)
 			{
+				char *initmsg = "*** System and Process Activity since Boot ***";
+				char *viewmsg;
+
+				if (rawreadflag)
+				{
+					viewmsg   = "Rawfile view";
+				}
+				else
+				{
+ 					uid_t ruid, euid, suid;
+
+					getresuid(&ruid, &euid, &suid);
+
+					if (suid == 0)
+					{
+						viewmsg   = "Unrestricted view (privileged)";
+					}
+					else
+					{
+						viewmsg   = "Restricted view (unprivileged)";
+					}
+				}
+
 				if (screen)
 				{
 					if (usecolors)
@@ -651,15 +462,15 @@ generic_samp(time_t curtime, int nsecs,
 
 					attron(A_BLINK);
 
-					printg("%*s", (COLS-45)/2, " ");
+					printg("%*s",
+						(COLS-strlen(initmsg)-strlen(viewmsg)-5)/2, " ");
 				}
 				else
 				{
-					printg("                   ");
+					printg("        ");
 				}
 
-       				printg("*** system and process activity "
-				       "since boot ***");
+       				printg(initmsg);
 
 				if (screen)
 				{
@@ -667,6 +478,26 @@ generic_samp(time_t curtime, int nsecs,
 						attroff(COLOR_PAIR(COLORINFO));
 					attroff(A_BLINK);
 				}
+
+				printg("     ");
+
+				if (screen)
+				{
+					if (usecolors)
+						attron(COLOR_PAIR(COLORALMOST));
+
+					attron(A_BLINK);
+				}
+
+       				printg(viewmsg);
+
+				if (screen)
+				{
+					if (usecolors)
+						attroff(COLOR_PAIR(COLORALMOST));
+					attroff(A_BLINK);
+				}
+
 			}
 		}
 
@@ -827,6 +658,7 @@ generic_samp(time_t curtime, int nsecs,
 			if ( procsel.userid[0] == USERSTUB &&
 			    !procsel.prognamesz            &&
 			    !procsel.container[0]          &&
+			    !procsel.states[0]             &&
 			    !procsel.argnamesz             &&
 			    !procsel.pid[0]                &&
 			    !suppressexit                    )
@@ -1003,7 +835,8 @@ generic_samp(time_t curtime, int nsecs,
 
 			priphead(firstproc/plistsz+1, (ncurlist-1)/plistsz+1,
 			       		&showtype, &curorder,
-					showorder == MSORTAUTO ? 1 : 0);
+					showorder == MSORTAUTO ? 1 : 0,
+					sstat->cpu.nrcpu);
 
 			if (screen)
 			{
@@ -1748,6 +1581,81 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** focus on specific process/thread state
+			   */
+			   case MSELSTATE:
+				alarm(0);	/* stop the clock */
+				echo();
+
+				move(statline, 0);
+				clrtoeol();
+
+				/* Linux fs/proc/array.c - task_state_array */
+				printw("Comma-separated process/thread states "
+				       "(R|S|D|I|T|t|X|Z|P): ");
+
+				memset(procsel.states, 0, sizeof procsel.states);
+
+				scanw("%15s\n", genline);
+
+				char *sp = strtok(genline, ",");
+
+				while (sp && *sp)
+				{
+					if (isspace(*sp))
+					{
+						sp++;
+						continue;
+					}
+
+					if (strlen(sp) > 1)
+					{
+						statmsg = "Invalid state!";
+						memset(procsel.states, 0,
+							sizeof procsel.states);
+						break;
+					}
+
+					int needed = 0;
+
+					switch (*sp)
+					{
+						case 'R': /* running */
+						case 'S': /* sleeping */
+						case 'D': /* disk sleep */
+						case 'I': /* idle */
+						case 'T': /* stopped */
+						case 't': /* tracing stop */
+						case 'X': /* dead */
+						case 'Z': /* zombie */
+						case 'P': /* parked */
+							if (!strchr(procsel.states, *sp))
+								needed = 1;
+							break;
+						default:
+							statmsg = "Invalid state!";
+							memset(procsel.states,
+								0, sizeof procsel.states);
+							beep();
+							break;
+					}
+
+					if (needed)
+					    procsel.states[strlen(procsel.states)] = *sp;
+
+					sp = strtok(NULL, ",");
+				}
+
+				noecho();
+
+				move(statline, 0);
+
+				if (interval && !paused && !rawreadflag)
+					alarm(3);  /* set short timer */
+
+				firstproc = 0;
+				break;
 
 			   /*
 			   ** focus on specific command line arguments
@@ -2153,6 +2061,11 @@ generic_samp(time_t curtime, int nsecs,
 				            "statistics (now %d): ",
 					    maxcontlines, statline);
 
+				maxnumalines =
+				  getnumval("Maximum lines for numa "
+				            "statistics (now %d): ",
+					    maxnumalines, statline);
+
 				if (interval && !paused && !rawreadflag)
 					alarm(3);  /* set short timer */
 
@@ -2591,6 +2504,15 @@ procsuppress(struct tstat *curstat, struct pselection *sel)
 		}
 	}
 
+	/*
+	** check if only processes in specific states should be shown 
+	*/
+	if (sel->states[0])
+	{
+		if (strchr(sel->states, curstat->gen.state) == NULL)
+			return 1;
+	}
+
 	return 0;
 }
 
@@ -2624,6 +2546,9 @@ limitedlines(void)
 
 	if (maxcontlines == 999)	// default?
 		maxcontlines = 1;
+
+	if (maxnumalines == 999)	// default?
+		maxnumalines = 0;
 }
 
 /*
@@ -2682,7 +2607,7 @@ printg(const char *format, ...)
 	va_start(args, format);
 
 	if (screen)
-		vwprintw(stdscr, (char *) format, args);
+		vw_printw(stdscr, (char *) format, args);
 	else
 		vprintf(format, args);
 
@@ -2983,10 +2908,11 @@ static struct helptext {
 	                              "(regular expression)\n", MSELUSER},
 	{"\t'%c'  - focus on specific program name        "
 	                              "(regular expression)\n", MSELPROC},
-	{"\t'%c'  - focus on specific contained id (CID)\n",    MSELCONT},
+	{"\t'%c'  - focus on specific container id (CID)\n",    MSELCONT},
 	{"\t'%c'  - focus on specific command line string "
 	                              "(regular expression)\n", MSELARG},
 	{"\t'%c'  - focus on specific process id (PID)\n",      MSELPID},
+	{"\t'%c'  - focus on specific process/thread state(s)\n", MSELSTATE},
 	{"\n",							' '},
 	{"System resource selections (keys shown in header line):\n",' '},
 	{"\t'%c'  - focus on specific system resources    "
@@ -3327,6 +3253,11 @@ do_maxcont(char *name, char *val)
 	maxcontlines = get_posval(name, val);
 }
 
+void
+do_maxnuma(char *name, char *val)
+{
+	maxnumalines = get_posval(name, val);
+}
 
 struct colmap {
 	char 	*colname;
