@@ -33,6 +33,8 @@ struct syscap {
 	count_t	availnet;
 	int	nrgpu;
 	count_t	availgpumem; 	// GPU memory in Kb!
+	int	nrmemnuma;
+	int	nrcpunuma;
 };
 
 struct pselection {
@@ -50,6 +52,8 @@ struct pselection {
 	regex_t	argregex;
 
 	char 	container[16];
+
+	char	states[16];
 };
 
 struct sselection {
@@ -107,12 +111,14 @@ struct sselection {
 #define	MSYSFIXED	'f'
 #define	MSYSNOSORT	'F'
 #define	MSYSLIMIT	'l'
+#define MRMSPACES	'Z'
 
 #define	MSELUSER	'U'
 #define	MSELPROC	'P'
 #define	MSELCONT	'J'
 #define	MSELPID		'I'
 #define	MSELARG		'/'
+#define	MSELSTATE	'Q'
 #define	MSELSYS		'S'
 
 #define	MALLPROC	'a'
@@ -150,7 +156,7 @@ void	showcmdproc(struct tstat *, double, int, int);
 
 void	printg     (const char *, ...);
 int	prisyst(struct sstat  *, int, int, int, int, struct sselection *,
-			char *, int, int, int, int, int, int, int, int, int);
+			char *, int, int, int, int, int, int, int, int, int, int);
 int	priproc(struct tstat  **, int, int, int, int, int, char, char,
 	        struct syscap *, int, int);
-void	priphead(int, int, char *, char *, char);
+void	priphead(int, int, char *, char *, char, count_t);
