@@ -390,14 +390,19 @@ sys_printdef *memsyspdefs[] = {
 	&syspdef_BLANKBOX,
 	&syspdef_HUPTOT,
 	&syspdef_HUPUSE,
+	&syspdef_NUMNUMA,
         0
 };
 sys_printdef *swpsyspdefs[] = {
 	&syspdef_SWPTOT,
 	&syspdef_SWPFREE,
-	&syspdef_BLANKBOX,
 	&syspdef_SWPCACHE,
 	&syspdef_BLANKBOX,
+	&syspdef_ZSWTOTAL,
+	&syspdef_ZSWSTORED,
+	&syspdef_KSMSHARING,
+	&syspdef_KSMSHARED,
+	&syspdef_BLANKBOX,
 	&syspdef_SWPCOMMITTED,
 	&syspdef_SWPCOMMITLIM,
 	&syspdef_BLANKBOX,
@@ -407,11 +412,43 @@ sys_printdef *pagsyspdefs[] = {
 	&syspdef_PAGSCAN,
 	&syspdef_PAGSTEAL,
 	&syspdef_PAGSTALL,
+	&syspdef_PAGCOMPACT,
+	&syspdef_PGMIGRATE,
+	&syspdef_NUMAMIGRATE,
 	&syspdef_PAGSWIN,
 	&syspdef_PAGSWOUT,
+	&syspdef_OOMKILLS,
 	&syspdef_BLANKBOX,
         0
 };
+sys_printdef *memnumasyspdefs[] = {
+	&syspdef_NUMATOT,
+	&syspdef_NUMAFREE,
+	&syspdef_NUMAFILEPAGE,
+	&syspdef_NUMANR,
+	&syspdef_NUMADIRTY,
+	&syspdef_NUMAACTIVE,
+	&syspdef_NUMAINACTIVE,
+	&syspdef_NUMASLAB,
+	&syspdef_NUMASLABRECLAIM,
+	&syspdef_NUMASHMEM,
+	&syspdef_NUMAFRAG,
+	&syspdef_NUMAHUPTOT,
+        0
+};
+sys_printdef *cpunumasyspdefs[] = {
+	&syspdef_NUMACPUSYS,
+	&syspdef_NUMACPUUSER,
+	&syspdef_NUMACPUNICE,
+	&syspdef_NUMACPUIRQ,
+	&syspdef_NUMACPUSOFTIRQ,
+	&syspdef_NUMACPUIDLE,
+	&syspdef_NUMACPUWAIT,
+	&syspdef_NUMACPUSTEAL,
+	&syspdef_NUMACPUGUEST,
+	&syspdef_NUMANUMCPU,
+        0
+};
 sys_printdef *psisyspdefs[] = {
 	&syspdef_PSICPUSTOT,
 	&syspdef_PSIMEMSTOT,
@@ -439,10 +476,12 @@ sys_printdef *dsksyspdefs[] = {
 	&syspdef_DSKBUSY,
 	&syspdef_DSKNREAD,
 	&syspdef_DSKNWRITE,
+	&syspdef_DSKNDISC,
 	&syspdef_DSKMBPERSECWR,
 	&syspdef_DSKMBPERSECRD,
-	&syspdef_DSKKBPERWR,
 	&syspdef_DSKKBPERRD,
+	&syspdef_DSKKBPERWR,
+	&syspdef_DSKKBPERDS,
 	&syspdef_DSKAVQUEUE,
 	&syspdef_DSKAVIO,
 	&syspdef_BLANKBOX,
@@ -669,6 +708,8 @@ sys_printpair cplline[MAXITEMS];
 sys_printpair gpuline[MAXITEMS];
 sys_printpair memline[MAXITEMS];
 sys_printpair swpline[MAXITEMS];
+sys_printpair memnumaline[MAXITEMS];
+sys_printpair cpunumaline[MAXITEMS];
 sys_printpair pagline[MAXITEMS];
 sys_printpair psiline[MAXITEMS];
 sys_printpair contline[MAXITEMS];
@@ -758,39 +799,53 @@ makeargv(char *line, const char *linename, name_prio *vec)
  * make_sys_prints: make array of sys_printpairs
  * input: string, sys_printpair array, maxentries
  */
-void
+static void
 make_sys_prints(sys_printpair *ar, int maxn, const char *pairs, 
-                sys_printdef *permissables[], const char *linename)
+                sys_printdef *permissables[], const char *linename,
+		struct sstat *sstat, extraparam *extra)
 {
-        name_prio items[MAXITEMS];
-        int n=strlen(pairs);
+        name_prio	items[MAXITEMS];
+        int		i, a, n=strlen(pairs);
+        char		str[n+1];
 
-        char str[n+1];
         strcpy(str, pairs);
 
         makeargv(str, linename, items);
 
-        int i;
-        for(i=0; items[i].name && i<maxn-1; ++i) 
+        for(i=a=0; items[i].name && i<maxn-1; i++) 
         {
                 const char *name=items[i].name;
                 int j;
+
                 for (j=0; permissables[j] != 0; ++j)
                 {
-                        if (strcmp(permissables[j]->configname, name)==0)
+                        if (strcmp(permissables[j]->configname, name) == 0)
                         {
-                                ar[i].f=permissables[j];
-                                ar[i].prio=items[i].prio;
+				// call validate function to see if this 
+				// counter is relevant
+				//
+				if (sstat != NULL && 
+				    permissables[j]->dovalidate != NULL &&
+				    permissables[j]->dovalidate(sstat) == 0)
+					break;
+
+                               	ar[a].f    = permissables[j];
+                               	ar[a].prio = items[i].prio;
+				a++;
                                 break;
                         }
                 }
+
                 if (permissables[j]==0)
+		{
 			mcleanstop(1,
 			"atoprc - own system line: item %s invalid in %s line\n",
 			name, linename);
+		}
         }
-        ar[i].f=0;
-        ar[i].prio=0;
+
+        ar[a].f=0;
+        ar[a].prio=0;
 }
 
 
@@ -799,8 +854,8 @@ make_sys_prints(sys_printpair *ar, int maxn, const char *pairs,
  * init_proc_prints: determine width of columns that are
  *                   dependent of dynamic values 
  */
-void 
-init_proc_prints()
+static void 
+init_proc_prints(count_t numcpu)
 {
 	int 	i, numdigits = 5;
 	char	linebuf[64];
@@ -837,6 +892,15 @@ init_proc_prints()
 			idprocpdefs[i]->head = p;
 		}
 	}
+
+	/*
+	** fill number of positions for the SORTITEM (percentage),
+	** depending on the number of CPUs (e.g. with 10+ CPUs a process
+	** can reach a CPU percentage of 1000% and with 100+ CPUs a
+	** CPU percentage of 10000%).
+	*/
+	procprt_SORTITEM.width =
+		snprintf(linebuf, sizeof linebuf, "%lld", numcpu*100) + 1;
 }
 
 /*
@@ -931,17 +995,41 @@ totalcap(struct syscap *psc, struct sstat *sstat,
 		psc->availgpumem += sstat->gpu.gpu[i].memtotnow;
 
 	psc->nrgpu = sstat->gpu.nrgpus;
+
+	psc->nrmemnuma = sstat->memnuma.nrnuma;
+	psc->nrcpunuma = sstat->cpunuma.nrnuma;
 }
 
 /*
 ** calculate cumulative system- and user-time for all active processes
+** besides, initialize all counter lines on system level
 */
 void
 pricumproc(struct sstat *sstat, struct devtstat *devtstat,
            int nexit, unsigned int noverflow, int avgval, int nsecs)
 {
+        static int 	firsttime=1;
+        int     	i;
+        extraparam 	extra;
 
-        static int firsttime=1;
+        for (i=0, extra.totut=extra.totst=0; i < devtstat->nprocactive; i++)
+        {
+		struct tstat *curstat = *(devtstat->procactive+i);
+
+                extra.totut	+= curstat->cpu.utime;
+                extra.totst 	+= curstat->cpu.stime;
+        }
+
+        extra.nproc	= devtstat->nprocall;
+	extra.ntrun	= devtstat->totrun;
+	extra.ntslpi	= devtstat->totslpi;
+	extra.ntslpu	= devtstat->totslpu;
+        extra.nzomb	= devtstat->totzombie;
+        extra.nexit	= nexit;
+        extra.noverflow	= noverflow;
+        extra.avgval	= avgval;
+        extra.nsecs	= nsecs;
+        extra.index	= 0;
 
         if (firsttime)
         {
@@ -960,8 +1048,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                         "PRCNZOMBIE:5 "
                         "PRCCLONES:4 "
 	                "BLANKBOX:0 "
-                        "PRCNNEXIT:6", prcsyspdefs, "builtin sysprcline");
+                        "PRCNNEXIT:6",
+			prcsyspdefs, "builtin sysprcline",
+			sstat, &extra);
                 }
+
                 if (allcpuline[0].f == 0)
                 {
                     make_sys_prints(allcpuline, MAXITEMS,
@@ -977,7 +1068,9 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                         "CPUIPC:5 "
                         "CPUCYCLE:4 "
                         "CPUFREQ:4 "
-                        "CPUSCALE:4 ", cpusyspdefs, "builtin allcpuline");
+                        "CPUSCALE:4 ",
+			cpusyspdefs, "builtin allcpuline",
+			sstat, &extra);
                 }
 
                 if (indivcpuline[0].f == 0)
@@ -995,7 +1088,9 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                         "CPUIIPC:5 "
                         "CPUICYCLE:4 "
                         "CPUIFREQ:4 "
-                        "CPUISCALE:4 ", cpisyspdefs, "builtin indivcpuline");
+                        "CPUISCALE:4 ",
+			cpisyspdefs, "builtin indivcpuline",
+			sstat, &extra);
                 }
 
                 if (cplline[0].f == 0)
@@ -1008,7 +1103,9 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "CPLCSW:6 "
 	                "CPLINTR:5 "
 	                "BLANKBOX:0 "
-	                "CPLNUMCPU:1", cplsyspdefs, "builtin cplline");
+	                "CPLNUMCPU:1",
+			cplsyspdefs, "builtin cplline",
+			sstat, &extra);
                 }
 
                 if (gpuline[0].f == 0)
@@ -1023,7 +1120,9 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "GPUMEMAVG:2 "
 	                "GPUNRPROC:2 "
 	                "BLANKBOX:0 "
-	                "GPUTYPE:1 ", gpusyspdefs, "builtin gpuline");
+	                "GPUTYPE:1 ",
+			gpusyspdefs, "builtin gpuline",
+			NULL, NULL);
                 }
 
                 if (memline[0].f == 0)
@@ -1046,35 +1145,86 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "ZFSARC:6 "
 	                "BLANKBOX:0 "
 	                "HUPTOT:6 "
-	                "HUPUSE:3 ", memsyspdefs, "builtin memline");
+	                "HUPUSE:3 "
+	                "NUMNUMA:7 ",
+			memsyspdefs, "builtin memline",
+			sstat, &extra);
                 }
+
                 if (swpline[0].f == 0)
                 {
                     make_sys_prints(swpline, MAXITEMS,
-	                "SWPTOT:3 "
-	                "SWPFREE:4 "
+	                "SWPTOT:5 "
+	                "SWPFREE:6 "
+	                "SWPCACHE:4 "
 	                "BLANKBOX:0 "
+	                "ZSWTOTAL:3 "
+	                "ZSWSTORED:3 "
 	                "BLANKBOX:0 "
-	                "SWPCACHE:2 "
+	                "KSMSHARED:2 "
+	                "KSMSHARING:2 "
 	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
-	                "SWPCOMMITTED:5 "
-	                "SWPCOMMITLIM:6", swpsyspdefs, "builtin swpline");
+	                "SWPCOMMITTED:7 "
+	                "SWPCOMMITLIM:8",
+ 			swpsyspdefs, "builtin swpline",
+			sstat, &extra);
                 }
+
+                if (memnumaline[0].f == 0)
+                {
+                    make_sys_prints(memnumaline, MAXITEMS,
+	                "NUMATOT:8 "
+	                "NUMAFREE:9 "
+	                "NUMAFILEPAGE:9 "
+	                "NUMANR:7 "
+	                "NUMADIRTY:5 "
+	                "NUMAACTIVE:5 "
+	                "NUMAINACTIVE:5 "
+	                "NUMASLAB:7 "
+	                "NUMASLABRECLAIM:4 "
+	                "NUMASHMEM:4 "
+	                "NUMAFRAG:6 "
+	                "NUMAHUPTOT:3 ",
+			memnumasyspdefs, "builtin memnumaline",
+			sstat, &extra);
+                }
+
+                if (cpunumaline[0].f == 0)
+                {
+                    make_sys_prints(cpunumaline, MAXITEMS,
+	                "NUMACPUSYS:9 "
+	                "NUMACPUUSER:8 "
+	                "NUMACPUNICE:8 "
+	                "NUMACPUIRQ:6 "
+	                "NUMACPUSOFTIRQ:6 "
+	                "NUMACPUIDLE:7 "
+	                "NUMACPUWAIT:7 "
+	                "NUMACPUSTEAL:2 "
+	                "NUMACPUGUEST:3 "
+	                "NUMANUMCPU:5",
+			cpunumasyspdefs, "builtin cpunumaline",
+			NULL, NULL);
+                }
+
                 if (pagline[0].f == 0)
                 {
                     make_sys_prints(pagline, MAXITEMS,
 	                "PAGSCAN:3 "
-	                "PAGSTEAL:3 "
+	                "PAGSTEAL:2 "
 	                "PAGSTALL:1 "
+	                "PAGCOMPACT:5 "
+			"NUMAMIGRATE:5"
+			"PGMIGRATE:6"
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
-	                "PAGSWIN:3 "
-	                "PAGSWOUT:4", pagsyspdefs, "builtin pagline");
+	                "PAGSWIN:5 "
+	                "PAGSWOUT:7 "
+			"OOMKILLS:8 ",
+			pagsyspdefs, "builtin pagline",
+			sstat, &extra);
                 }
+
                 if (psiline[0].f == 0)
                 {
                     make_sys_prints(psiline, MAXITEMS,
@@ -1088,8 +1238,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "PSIMEMF:3 "
 	                "PSIIOS:4 "
 	                "PSIIOF:2 "
-	                "BLANKBOX:0 ", psisyspdefs, "builtin psiline");
+	                "BLANKBOX:0 ",
+			psisyspdefs, "builtin psiline",
+			sstat, &extra);
                 }
+
                 if (contline[0].f == 0)
                 {
                     make_sys_prints(contline, MAXITEMS,
@@ -1098,22 +1251,30 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "CONTCPU:6 "
 	                "CONTMEM:6 "
 	                "BLANKBOX:0 "
-	                "BLANKBOX:0 ", contsyspdefs, "builtin contline");
+	                "BLANKBOX:0 ",
+			contsyspdefs, "builtin contline",
+			NULL, NULL);
                 }
+
                 if (dskline[0].f == 0)
                 {
                     make_sys_prints(dskline, MAXITEMS,
-	                "DSKNAME:8 "
-	                "DSKBUSY:7 "
-	                "DSKNREAD:6 "
-	                "DSKNWRITE:6 "
-	                "DSKKBPERRD:4 "
-	                "DSKKBPERWR:4 "
-                        "DSKMBPERSECRD:5 "
-                        "DSKMBPERSECWR:5 "
+	                "DSKNAME:9 "
+	                "DSKBUSY:8 "
+	                "DSKNREAD:7 "
+	                "DSKNWRITE:7 "
+	                "DSKNDISC:6 "
+	                "DSKKBPERRD:5 "
+	                "DSKKBPERWR:5 "
+	                "DSKKBPERDS:4 "
+                        "DSKMBPERSECRD:6 "
+                        "DSKMBPERSECWR:6 "
 	                "DSKAVQUEUE:1 "
-	                "DSKAVIO:5", dsksyspdefs, "builtin dskline");
+	                "DSKAVIO:6",
+			dsksyspdefs, "builtin dskline",
+			sstat, &extra);
                 }
+
                 if (nfsmountline[0].f == 0)
                 {
                     make_sys_prints(nfsmountline, MAXITEMS,
@@ -1131,8 +1292,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 			"NFMMREAD:3 "
 			"NFMMWRITE:2 "
 	                "BLANKBOX:0 "
-                        "BLANKBOX:0", nfsmntsyspdefs, "builtin nfsmountline");
+                        "BLANKBOX:0",
+			nfsmntsyspdefs, "builtin nfsmountline",
+			NULL, NULL);
                 }
+
                 if (nfcline[0].f == 0)
                 {
                     make_sys_prints(nfcline, MAXITEMS,
@@ -1145,8 +1309,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
-	                "BLANKBOX:0 ", nfcsyspdefs, "builtin nfcline");
+	                "BLANKBOX:0 ",
+			nfcsyspdefs, "builtin nfcline",
+			sstat, &extra);
 		}
+
                 if (nfsline[0].f == 0)
                 {
                     make_sys_prints(nfsline, MAXITEMS,
@@ -1166,8 +1333,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "BLANKBOX:0 "
 	                "NFSBADFMT:4 "
 	                "NFSBADAUT:4 "
-	                "NFSBADCLN:4 ", nfssyspdefs, "builtin nfsline");
+	                "NFSBADCLN:4 ",
+			nfssyspdefs, "builtin nfsline",
+			sstat, &extra);
 		}
+
                 if (nettransportline[0].f == 0)
                 {
                     make_sys_prints(nettransportline, MAXITEMS,
@@ -1182,8 +1352,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                         "NETTCPINERR:3 "
                         "NETTCPORESET:2 "
                         "NETUDPNOPORT:1 "
-                        "NETUDPINERR:3", nettranssyspdefs, "builtin nettransportline");
+                        "NETUDPINERR:3",
+			nettranssyspdefs, "builtin nettransportline",
+			sstat, &extra);
                 }
+
                 if (netnetline[0].f == 0)
                 {
                     make_sys_prints(netnetline, MAXITEMS,
@@ -1196,8 +1369,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
                         "NETICMPIN:1 "
-                        "NETICMPOUT:1 ", netnetsyspdefs, "builtin netnetline");
+                        "NETICMPOUT:1 ",
+			netnetsyspdefs, "builtin netnetline",
+			sstat, &extra);
                 }
+
                 if (netinterfaceline[0].f == 0)
                 {
                     make_sys_prints(netinterfaceline, MAXITEMS,
@@ -1215,8 +1391,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                         "NETRCVERR:4 "
                         "NETSNDERR:4 "
                         "NETRCVDROP:3 "
-                        "NETSNDDROP:3", netintfsyspdefs, "builtin netinterfaceline");
+                        "NETSNDDROP:3",
+			netintfsyspdefs, "builtin netinterfaceline",
+			NULL, NULL);
                 }
+
                 if (infinibandline[0].f == 0)
                 {
                     make_sys_prints(infinibandline, MAXITEMS,
@@ -1234,33 +1413,12 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
-	                "BLANKBOX:0 ", infinisyspdefs, "builtin infinibandline");
+	                "BLANKBOX:0 ",
+			infinisyspdefs, "builtin infinibandline",
+			NULL, NULL);
                 }
         }  // firsttime
 
-
-        int     i;
-        extraparam extra;
-
-
-        for (i=0, extra.totut=extra.totst=0; i < devtstat->nprocactive; i++)
-        {
-		struct tstat *curstat = *(devtstat->procactive+i);
-
-                extra.totut	+= curstat->cpu.utime;
-                extra.totst 	+= curstat->cpu.stime;
-        }
-
-        extra.nproc	= devtstat->nprocall;
-	extra.ntrun	= devtstat->totrun;
-	extra.ntslpi	= devtstat->totslpi;
-	extra.ntslpu	= devtstat->totslpu;
-        extra.nzomb	= devtstat->totzombie;
-        extra.nexit	= nexit;
-        extra.noverflow	= noverflow;
-        extra.avgval	= avgval;
-        extra.nsecs	= nsecs;
-
         move(1, 0);
         showsysline(sysprcline, sstat, &extra, "PRC", 0);
 }
@@ -1270,7 +1428,7 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 */
 void
 priphead(int curlist, int totlist, char *showtype, char *showorder,
-							char autosort)
+					char autosort, count_t numcpu)
 {
         static int      firsttime=1;
         static int      prev_supportflags = -1, prev_threadview = -1;
@@ -1281,7 +1439,7 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 	*/
         if (firsttime) 
         {
-		init_proc_prints();
+		init_proc_prints(numcpu);
 
                 make_proc_prints(memprocs, MAXITEMS, 
                         "PID:10 TID:3 MINFLT:2 MAJFLT:2 VSTEXT:4 VSLIBS:4 "
@@ -1677,7 +1835,7 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
         int fixedhead, struct sselection *selp, char *highorderp,
         int maxcpulines, int maxgpulines, int maxdsklines, int maxmddlines,
 	int maxlvmlines, int maxintlines, int maxifblines,
-	int maxnfslines, int maxcontlines)
+	int maxnfslines, int maxcontlines, int maxnumalines)
 {
         extraparam      extra;
         int             lin;
@@ -1896,15 +2054,112 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
         showsysline(swpline, sstat, &extra, "SWP", badness);
         curline++;
 
+	/*
+	** memory info related for per NUMA
+	*/
+	if (sstat->memnuma.nrnuma > 1)
+	{
+		for (extra.index=lin=0;
+		     extra.index < sstat->memnuma.nrnuma && lin < maxnumalines;
+		     extra.index++)
+		{
+			busy = (sstat->memnuma.numa[extra.index].totmem
+				- sstat->memnuma.numa[extra.index].freemem
+				- sstat->memnuma.numa[extra.index].filepage
+				- sstat->memnuma.numa[extra.index].slabreclaim
+				+ sstat->memnuma.numa[extra.index].shmem)
+					* 100.0 / sstat->memnuma.numa[extra.index].totmem;
+
+			if (membadness)
+				badness = busy * 100 / membadness;
+			else
+				badness = 0;
+
+			if (highbadness < badness)
+			{
+				highbadness = badness;
+				*highorderp = MSORTMEM;
+			}
+
+			if (screen)
+				move(curline, 0);
+
+			showsysline(memnumaline, sstat, &extra, "NUM", badness);
+			curline++;
+			lin++;
+		}
+	}
+
+	/*
+	** Accumulate each cpu statistic for per NUMA
+	*/
+	if (sstat->cpunuma.nrnuma > 1)
+	{
+		for (extra.index=lin=0;
+		     extra.index < sstat->cpunuma.nrnuma && lin < maxnumalines;
+		     extra.index++)
+		{
+			extra.pernumacputot = sstat->cpunuma.numa[extra.index].stime +
+					      sstat->cpunuma.numa[extra.index].utime +
+					      sstat->cpunuma.numa[extra.index].ntime +
+					      sstat->cpunuma.numa[extra.index].itime +
+					      sstat->cpunuma.numa[extra.index].wtime +
+					      sstat->cpunuma.numa[extra.index].Itime +
+					      sstat->cpunuma.numa[extra.index].Stime +
+					      sstat->cpunuma.numa[extra.index].steal;
+
+			if (extra.pernumacputot ==
+				(sstat->cpunuma.numa[extra.index].itime +
+				 sstat->cpunuma.numa[extra.index].wtime  ) &&
+				 !fixedhead                             )
+				continue;       /* inactive cpu */
+
+			if (extra.pernumacputot == 0)
+				extra.pernumacputot = 1; /* avoid divide-by-zero */
+
+			busy = (extra.pernumacputot -
+					sstat->cpunuma.numa[extra.index].itime -
+					sstat->cpunuma.numa[extra.index].wtime)
+					* 100.0 / extra.pernumacputot;
+
+			if (cpubadness)
+				badness = busy * 100 / cpubadness;
+			else
+				badness = 0;
+
+			if (highbadness < badness)
+			{
+				highbadness = badness;
+				*highorderp = MSORTCPU;
+			}
+
+			extra.percputot = extra.pernumacputot /
+						(sstat->cpu.nrcpu/sstat->cpunuma.nrnuma);
+			if (extra.percputot == 0)
+				extra.percputot = 1; /* avoid divide-by-zero */
+
+			if (screen)
+				move(curline, 0);
+
+			showsysline(cpunumaline, sstat, &extra, "NUC", badness);
+			curline++;
+			lin++;
+		}
+	}
+
         /*
         ** PAGING statistics
         */
-        if (fixedhead             ||
-            sstat->mem.pgscans    ||
-            sstat->mem.pgsteal    ||
-            sstat->mem.allocstall ||
-            sstat->mem.swins      ||
-            sstat->mem.swouts       )
+        if (fixedhead             	||
+            sstat->mem.pgscans    	||
+            sstat->mem.pgsteal    	||
+            sstat->mem.allocstall 	||
+            sstat->mem.compactstall 	||
+            sstat->mem.swins      	||
+            sstat->mem.swouts     	||
+            sstat->mem.oomkills > 0   	||
+            sstat->mem.pgmigrate     	||
+            sstat->mem.numamigrate     	  )
         {
                 busy = sstat->mem.swouts / nsecs * pagbadness;
 
@@ -1933,7 +2188,7 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 		if (screen)
                 	move(curline, 0);
 
-                showsysline(pagline, sstat, &extra,"PAG", badness);
+                showsysline(pagline, sstat, &extra, "PAG", badness);
                 curline++;
         }
 
@@ -2638,6 +2893,40 @@ contcompar(const void *a, const void *b)
         return  0;
 }
 
+int
+memnumacompar(const void *a, const void *b)
+{
+        register count_t aused = ((struct mempernuma *)a)->totmem -
+                                 ((struct mempernuma *)a)->freemem;
+        register count_t bused = ((struct mempernuma *)b)->totmem -
+                                 ((struct mempernuma *)b)->freemem;
+
+        if (aused < bused)
+                return  1;
+
+        if (aused > bused)
+                return -1;
+
+        return  0;
+}
+
+int
+cpunumacompar(const void *a, const void *b)
+{
+        register count_t aidle = ((struct cpupernuma *)a)->itime +
+                                 ((struct cpupernuma *)a)->wtime;
+        register count_t bidle = ((struct cpupernuma *)b)->itime +
+                                 ((struct cpupernuma *)b)->wtime;
+
+        if (aidle < bidle)
+                return -1;
+
+        if (aidle > bidle)
+                return  1;
+
+	return  0;
+}
+
 /*
 ** handle modifications from the /etc/atoprc and ~/.atoprc file
 */
@@ -2730,79 +3019,106 @@ do_almostcrit(char *name, char *val)
 void
 do_ownsysprcline(char *name, char *val)
 {
-        make_sys_prints(sysprcline, MAXITEMS, val, prcsyspdefs, name);
+        make_sys_prints(sysprcline, MAXITEMS, val, prcsyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownallcpuline(char *name, char *val)
 {
-        make_sys_prints(allcpuline, MAXITEMS, val, cpusyspdefs, name);
+        make_sys_prints(allcpuline, MAXITEMS, val, cpusyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownindivcpuline(char *name, char *val)
 {
-        make_sys_prints(indivcpuline, MAXITEMS, val, cpisyspdefs, name);
+        make_sys_prints(indivcpuline, MAXITEMS, val, cpisyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_owncplline(char *name, char *val)
 {
-        make_sys_prints(cplline, MAXITEMS, val, cplsyspdefs, name);
+        make_sys_prints(cplline, MAXITEMS, val, cplsyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_owngpuline(char *name, char *val)
 {
-        make_sys_prints(gpuline, MAXITEMS, val, gpusyspdefs, name);
+        make_sys_prints(gpuline, MAXITEMS, val, gpusyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownmemline(char *name, char *val)
 {
-        make_sys_prints(memline, MAXITEMS, val, memsyspdefs, name);
+        make_sys_prints(memline, MAXITEMS, val, memsyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownswpline(char *name, char *val)
 {
-        make_sys_prints(swpline, MAXITEMS, val, swpsyspdefs, name);
+        make_sys_prints(swpline, MAXITEMS, val, swpsyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownpagline(char *name, char *val)
 {
-        make_sys_prints(pagline, MAXITEMS, val, pagsyspdefs, name);
+        make_sys_prints(pagline, MAXITEMS, val, pagsyspdefs, name,
+					NULL, NULL);
+}
+
+void
+do_ownmemnumaline(char *name, char *val)
+{
+        make_sys_prints(memnumaline, MAXITEMS, val, memnumasyspdefs, name,
+					NULL, NULL);
+}
+
+void
+do_owncpunumaline(char *name, char *val)
+{
+        make_sys_prints(cpunumaline, MAXITEMS, val, cpunumasyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_owndskline(char *name, char *val)
 {
-        make_sys_prints(dskline, MAXITEMS, val, dsksyspdefs, name);
+        make_sys_prints(dskline, MAXITEMS, val, dsksyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownnettransportline(char *name, char *val)
 {
-        make_sys_prints(nettransportline, MAXITEMS, val, nettranssyspdefs, name);
+        make_sys_prints(nettransportline, MAXITEMS, val, nettranssyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownnetnetline(char *name, char *val)
 {
-        make_sys_prints(netnetline, MAXITEMS, val, netnetsyspdefs, name);
+        make_sys_prints(netnetline, MAXITEMS, val, netnetsyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_ownnetinterfaceline(char *name, char *val)
 {
-        make_sys_prints(netinterfaceline, MAXITEMS, val, netintfsyspdefs, name);
+        make_sys_prints(netinterfaceline, MAXITEMS, val, netintfsyspdefs, name,
+					NULL, NULL);
 }
 
 void
 do_owninfinibandline(char *name, char *val)
 {
-        make_sys_prints(infinibandline, MAXITEMS, val, infinisyspdefs, name);
+        make_sys_prints(infinibandline, MAXITEMS, val, infinisyspdefs, name,
+					NULL, NULL);
 }
 
 void
