@@ -27,59 +27,6 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
-**
-** $Log: showprocs.c,v $
-** Revision 1.15  2011/09/05 11:44:16  gerlof
-** *** empty log message ***
-**
-** Revision 1.14  2010/12/01 09:05:38  gerlof
-** Added a dash in the column PPID for exited processes.
-**
-** Revision 1.13  2010/11/12 06:11:58  gerlof
-** Sometimes segmentation-fault on particular CPU-types
-** due to memcpy i.s.o. memmove when moving memory in overlap.
-**
-** Revision 1.12  2010/04/23 14:06:42  gerlof
-** Added special routined for uid/gid not available for exited processes.
-**
-** Revision 1.11  2010/01/16 12:54:08  gerlof
-** Minor change for CPUNR.
-**
-** Revision 1.10  2010/01/16 11:37:25  gerlof
-** Corrected counters for patched kernels (JC van Winkel).
-**
-** Revision 1.9  2010/01/08 13:44:47  gerlof
-** Added policies batch, iso and idle for scheduling class.
-**
-** Revision 1.8  2010/01/08 11:25:13  gerlof
-** Corrected column-width and priorities of network-stats.
-**
-** Revision 1.7  2010/01/03 18:26:53  gerlof
-** Consistent naming of columns for process-related info.
-**
-** Revision 1.6  2009/12/19 21:03:21  gerlof
-** Alignment of CMD column (JC van Winkel).
-**
-** Revision 1.5  2009/12/12 10:11:49  gerlof
-** Register and display end date and end time for process.
-**
-** Revision 1.4  2009/12/12 09:05:56  gerlof
-** Corrected cumulated disk I/O per user/program (JC van Winkel).
-**
-** Revision 1.3  2009/12/10 14:01:52  gerlof
-** Add EUID, SUID and FSUID (and similar for GID's).
-**
-** Revision 1.2  2009/12/10 11:56:22  gerlof
-** Various bug-solutions.
-**
-** Revision 1.1  2009/12/10 09:31:23  gerlof
-** Initial revision
-**
-** Initial revision
-**
-**
-** Initial
-**
 */
 
 #include <sys/types.h>
@@ -294,10 +241,13 @@ showhdrline(proc_printpair* elemptr, int curlist, int totlist,
 
         while ((curelem=*elemptr).f!=0) 
         {
+		int widen = 0;
+
                 if (curelem.f->head==0)     // empty header==special: SORTITEM
                 {
-                        chead=columnhead[order];
-                        autoindic= autosort ? "A" : " ";
+                        chead     = columnhead[order];
+                        autoindic = autosort ? "A" : " ";
+			widen     = procprt_SORTITEM.width-3;
                 } 
                 else 
                 {
@@ -307,8 +257,8 @@ showhdrline(proc_printpair* elemptr, int curlist, int totlist,
 
                 if (screen)
                 {
-                        col += sprintf(buf+col, "%s%s%*s", autoindic, chead,
-                              colspacings[n], "");
+                        col += sprintf(buf+col, "%*s%s%*s",
+				widen, autoindic, chead, colspacings[n], "");
                 }
                 else
                 {
@@ -384,7 +334,7 @@ showprocline(proc_printpair* elemptr, struct tstat *curstat,
 
                 if (curelem.f->head==0)                // empty string=sortitem
                 {
-                        printg("%3.0lf%%", perc);    // cannot pass perc
+                        printg("%*.0lf%%", procprt_SORTITEM.width-1, perc);
                 }
                 else if (curstat->gen.state != 'E')  // active process
                 {
@@ -617,7 +567,7 @@ procprt_VGROW_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vgrow*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vgrow*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -635,7 +585,7 @@ procprt_RGROW_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.rgrow*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.rgrow*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -677,7 +627,7 @@ procprt_VSTEXT_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vexec*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vexec*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -695,7 +645,7 @@ procprt_VSIZE_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vmem*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vmem*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -713,7 +663,7 @@ procprt_RSIZE_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.rmem*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.rmem*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -734,7 +684,7 @@ procprt_PSIZE_a(struct tstat *curstat, int avgval, int nsecs)
 	if (curstat->mem.pmem == (unsigned long long)-1LL)	
         	return "    ?K";
 
-       	val2memstr(curstat->mem.pmem*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.pmem*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -752,7 +702,7 @@ procprt_VSLIBS_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vlibs*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vlibs*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -770,7 +720,7 @@ procprt_VDATA_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vdata*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vdata*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -788,7 +738,7 @@ procprt_VSTACK_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vstack*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vstack*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -806,7 +756,7 @@ procprt_SWAPSZ_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->mem.vswap*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->mem.vswap*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -1470,7 +1420,7 @@ char *
 procprt_RDDSK_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
-        val2memstr(curstat->dsk.rsz*512, buf, KBFORMAT, avgval, nsecs);
+        val2memstr(curstat->dsk.rsz*512, buf, BFORMAT, avgval, nsecs);
 
         return buf;
 }
@@ -1489,7 +1439,7 @@ procprt_WRDSK_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
 
-        val2memstr(curstat->dsk.wsz*512, buf, KBFORMAT, avgval, nsecs);
+        val2memstr(curstat->dsk.wsz*512, buf, BFORMAT, avgval, nsecs);
 
         return buf;
 }
@@ -1514,7 +1464,7 @@ procprt_CWRDSK_a(struct tstat *curstat, int avgval, int nsecs)
 	else
 		nett_wsz = 0;
 
-        val2memstr(nett_wsz*512, buf, KBFORMAT, avgval, nsecs);
+        val2memstr(nett_wsz*512, buf, BFORMAT, avgval, nsecs);
 
         return buf;
 }
@@ -1526,7 +1476,7 @@ char *
 procprt_WCANCEL_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[10];
-        val2memstr(curstat->dsk.cwsz*512, buf, KBFORMAT, avgval, nsecs);
+        val2memstr(curstat->dsk.cwsz*512, buf, BFORMAT, avgval, nsecs);
 
         return buf;
 }
@@ -1974,7 +1924,7 @@ procprt_GPUMEMNOW_ae(struct tstat *curstat, int avgval, int nsecs)
 	if (!curstat->gpu.state)
 		return "     -";
 
-        val2memstr(curstat->gpu.memnow*1024, buf, KBFORMAT, 0, 0);
+        val2memstr(curstat->gpu.memnow*1024, buf, BFORMAT, 0, 0);
         return buf;
 }
 
@@ -1993,7 +1943,7 @@ procprt_GPUMEMAVG_ae(struct tstat *curstat, int avgval, int nsecs)
 		return("    0K");
 
        	val2memstr(curstat->gpu.nrgpus * curstat->gpu.memcum /
-	           curstat->gpu.sample*1024, buf, KBFORMAT, 0, 0);
+	           curstat->gpu.sample*1024, buf, BFORMAT, 0, 0);
        	return buf;
 }
 
@@ -2059,7 +2009,7 @@ procprt_WCHAN_e(struct tstat *curstat, int avgval, int nsecs)
 }
 
 proc_printdef procprt_WCHAN =
-   { "WCHAN          ", "WCHAN", procprt_WCHAN_a, procprt_WCHAN_e, 15 };
+   { "WCHAN          ", "WCHAN", procprt_WCHAN_a, procprt_WCHAN_e, 15};
 /***************************************************************/
 char *
 procprt_RUNDELAY_a(struct tstat *curstat, int avgval, int nsecs)
@@ -2080,7 +2030,7 @@ procprt_RUNDELAY_e(struct tstat *curstat, int avgval, int nsecs)
 }
 
 proc_printdef procprt_RUNDELAY =
-   { "RDELAY", "RDELAY", procprt_RUNDELAY_a, procprt_RUNDELAY_e, 6 };
+   { "RDELAY", "RDELAY", procprt_RUNDELAY_a, procprt_RUNDELAY_e, 6};
 /***************************************************************/
 char *
 procprt_SORTITEM_ae(struct tstat *curstat, int avgval, int nsecs)
@@ -2088,5 +2038,5 @@ procprt_SORTITEM_ae(struct tstat *curstat, int avgval, int nsecs)
         return "";   // dummy function
 }
 
-proc_printdef procprt_SORTITEM = 
-   { 0, "SORTITEM", procprt_SORTITEM_ae, procprt_SORTITEM_ae, 4 };
+proc_printdef procprt_SORTITEM =   // width is dynamically defined!
+   { 0, "SORTITEM", procprt_SORTITEM_ae, procprt_SORTITEM_ae, 4};
