@@ -486,11 +486,16 @@ val2Hzstr(count_t value, char *strvalue)
 #define	ONETBYTE	1099511627776LL
 #define	ONEPBYTE	1125899906842624LL
 
-#define	MAXBYTE		1024
-#define	MAXKBYTE	ONEKBYTE*99999L
+#define	MAXBYTE		999
+#define	MAXKBYTE	ONEKBYTE*999L
+#define	MAXKBYTE9	ONEKBYTE*9L
 #define	MAXMBYTE	ONEMBYTE*999L
+#define	MAXMBYTE9	ONEMBYTE*9L
 #define	MAXGBYTE	ONEGBYTE*999LL
+#define	MAXGBYTE9	ONEGBYTE*9LL
 #define	MAXTBYTE	ONETBYTE*999LL
+#define	MAXTBYTE9	ONETBYTE*9LL
+#define	MAXPBYTE9	ONEPBYTE*9LL
 
 char *
 val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
@@ -516,31 +521,71 @@ val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 	*/
 	if (avgval && nsecs)
 	{
-		value     /= nsecs;
-		verifyval  = verifyval * 100 /nsecs;
-		basewidth -= 2;
-		suffix     = "/s";
-	}
-	
-	/*
-	** determine which format will be used on bases of the value itself
-	*/
-	if (verifyval <= MAXBYTE)	/* bytes ? */
-		aformat = ANYFORMAT;
-	else
-		if (verifyval <= MAXKBYTE)	/* kbytes ? */
-			aformat = KBFORMAT;
+		value         = llround((double)((double)value/(double)nsecs));
+		verifyval     = llround((double)((double)verifyval/(double)nsecs));
+		basewidth    -= 2;
+		suffix        = "/s";
+
+		if (verifyval <= MAXBYTE)	/* bytes ? */
+			aformat = BFORMAT;
 		else
-			if (verifyval <= MAXMBYTE)	/* mbytes ? */
-				aformat = MBFORMAT;
+			if (verifyval <= MAXKBYTE9)	/* kbytes 1-9 ? */
+				aformat = KBFORMAT;
 			else
-				if (verifyval <= MAXGBYTE)	/* gbytes ? */
-					aformat = GBFORMAT;
+				if (verifyval <= MAXKBYTE)	/* kbytes ? */
+					aformat = KBFORMAT_INT;
 				else
-				 	if (verifyval <= MAXTBYTE)/* tbytes? */
-						aformat = TBFORMAT;/* tbytes! */
+					if (verifyval <= MAXMBYTE9)	/* mbytes 1-9 ? */
+						aformat = MBFORMAT;
 					else
-						aformat = PBFORMAT;/* pbytes! */
+						if (verifyval <= MAXMBYTE)	/* mbytes 10-999 ? */
+							aformat = MBFORMAT_INT;
+						else
+							if (verifyval <= MAXGBYTE9)	/* gbytes 1-9 ? */
+								aformat = GBFORMAT;
+							else
+								if (verifyval <= MAXGBYTE)	/* gbytes 10-999 ? */
+									aformat = GBFORMAT_INT;
+								else
+									if (verifyval <= MAXTBYTE9)/* tbytes 1-9 ? */
+										aformat = TBFORMAT;/* tbytes! */
+									else
+										if (verifyval <= MAXTBYTE)/* tbytes 10-999? */
+											aformat = TBFORMAT_INT;/* tbytes! */
+										else
+											if (verifyval <= MAXPBYTE9)/* pbytes 1-9 ? */
+												aformat = PBFORMAT;/* pbytes! */
+											else
+												aformat = PBFORMAT_INT;/* pbytes! */
+
+	} else 
+	/*
+	** printed value per interval (normal mode) 
+	*/
+	{
+		/*
+		** determine which format will be used on bases of the value itself
+		*/
+		if (verifyval <= MAXBYTE)	/* bytes ? */
+			aformat = BFORMAT;
+		else
+			if (verifyval <= MAXKBYTE)	/* kbytes ? */
+				aformat = KBFORMAT;
+			else
+				if (verifyval <= MAXMBYTE)	/* mbytes ? */
+					aformat = MBFORMAT;
+				else
+					if (verifyval <= MAXGBYTE)	/* gbytes ? */
+						aformat = GBFORMAT;
+					else
+						if (verifyval <= MAXTBYTE)/* tbytes? */
+							aformat = TBFORMAT;/* tbytes! */
+						else
+							aformat = PBFORMAT;/* pbytes! */
+
+
+	}
+
 
 	/*
 	** check if this is also the preferred format
@@ -550,14 +595,19 @@ val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 
 	switch (aformat)
 	{
-	   case	ANYFORMAT:
-		sprintf(strvalue, "%*lld%s",
-				basewidth, value, suffix);
+	   case	BFORMAT:
+		sprintf(strvalue, "%*lldB%s",
+				basewidth-1, value, suffix);
 		break;
 
 	   case	KBFORMAT:
+		sprintf(strvalue, "%*.1lfK%s",
+			basewidth-1, (double)((double)value/ONEKBYTE), suffix); 
+		break;
+
+	   case	KBFORMAT_INT:
 		sprintf(strvalue, "%*lldK%s",
-				basewidth-1, value/ONEKBYTE, suffix);
+				basewidth-1, llround((double)((double)value/ONEKBYTE)), suffix);
 		break;
 
 	   case	MBFORMAT:
@@ -565,21 +615,41 @@ val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 			basewidth-1, (double)((double)value/ONEMBYTE), suffix); 
 		break;
 
+	   case	MBFORMAT_INT:
+		sprintf(strvalue, "%*lldM%s",
+			basewidth-1, llround((double)((double)value/ONEMBYTE)), suffix); 
+		break;
+
 	   case	GBFORMAT:
 		sprintf(strvalue, "%*.1lfG%s",
 			basewidth-1, (double)((double)value/ONEGBYTE), suffix);
 		break;
 
+	   case	GBFORMAT_INT:
+		sprintf(strvalue, "%*lldG%s",
+			basewidth-1, llround((double)((double)value/ONEGBYTE)), suffix);
+		break;
+
 	   case	TBFORMAT:
 		sprintf(strvalue, "%*.1lfT%s",
 			basewidth-1, (double)((double)value/ONETBYTE), suffix);
 		break;
 
+	   case	TBFORMAT_INT:
+		sprintf(strvalue, "%*lldT%s",
+			basewidth-1, llround((double)((double)value/ONETBYTE)), suffix);
+		break;
+
 	   case	PBFORMAT:
 		sprintf(strvalue, "%*.1lfP%s",
 			basewidth-1, (double)((double)value/ONEPBYTE), suffix);
 		break;
 
+	   case	PBFORMAT_INT:
+		sprintf(strvalue, "%*lldP%s",
+			basewidth-1, llround((double)((double)value/ONEPBYTE)), suffix);
+		break;
+
 	   default:
 		sprintf(strvalue, "!TILT!");
 	}
